/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /h/ellis/source/java/xcard/RCS/CardEngine.java,v 1.26 2001/09/28 04:13:48 ellis Exp $
 */

package xcard;

import java.io.*;
import java.net.URL;
import java.text.*;
import java.util.*;
import java.beans.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class CardEngine extends XCardBase {
    public static int NON_REPEATED = 0;
    public static int REPEATED = 1;

    private String[] series = {"excellent", "good", "average", "poor"};
    private int[] states = {State.EXCELLENT, State.GOOD, State.AVERAGE,
			    State.POOR};

    ArrayList sessionList = null;
    SessionEntry currentSession = null;

    CardList cardList = new CardList();
    CardListIterator itr = null;

    Entry currentEntry = null;
    int currentState = State.POOR;

    public CardEngine()
    {
	sessionList = new ArrayList();
	currentSession = new SessionEntry();
	sessionList.add(currentSession);
    }

    public void load(File filename)
	throws ParserConfigurationException, SAXException, IOException
    {
	if (filename != null && filename.exists()) {
	    cardList.clear();
	    sessionList = new ArrayList();

	    DocumentBuilderFactory docBuilderFactory = 
		DocumentBuilderFactory.newInstance();
	    docBuilderFactory.setValidating(false);
	    DocumentBuilder docBuilder = 
		docBuilderFactory.newDocumentBuilder();

	    Document doc = docBuilder.parse(filename);
	    processXML(doc);

	    itr = cardList.iterator();
	    itr.setCircular(true);
	    itr.setAllStatesVisible(false);

	    currentEntry = null;
	    currentState = initCurrentState();
	    fireXCardEvent("ENGINE_LOAD", this);
	}
    }

    public void load(InputStream is)
	throws ParserConfigurationException, SAXException, IOException
    {
	cardList.clear();
	sessionList = new ArrayList();

	DocumentBuilderFactory docBuilderFactory = 
	    DocumentBuilderFactory.newInstance();
	DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();

	Document doc = docBuilder.parse(is);
	processXML(doc);

	itr = cardList.iterator();
	itr.setCircular(true);
	itr.setAllStatesVisible(false);

	currentEntry = null;
	currentState = initCurrentState();
	fireXCardEvent("ENGINE_LOAD", this);
    }

    private int initCurrentState()
    {
	return State.POOR;
    }

    private void saveEntries(Writer os) throws IOException
    {
	Iterator iterator = cardList.iterator();
	StringBuffer buf = new StringBuffer();

	while (iterator.hasNext()) {
	    Entry entry = (Entry) iterator.next();
	    os.write("\n");
	    os.write("<entry>\n");

	    String word = entry.getWord();
	    if (word != null) {
		String prefix = entry.getPrefix();
		if (prefix != null && prefix.length() > 0) {
		    os.write("<word prefix=\"" + prefix + "\">");
		} else {
		    os.write("<word>");
		}
		os.write(word);
		os.write("</word>\n");
	    }

	    String translation = entry.getTranslation();
	    if (translation != null) {
		String transPrefix = entry.getTranslationPrefix();
		if (transPrefix != null && transPrefix.length() > 0) {
		    os.write("<trans prefix=\"" + transPrefix + "\">");
		} else {
		    os.write("<trans>");
		}
		os.write(translation);
		os.write("</trans>\n");
	    }

	    String comment = entry.getComment();
	    if (comment != null && comment.length() > 0) {
		os.write("<comment>");
		os.write(comment);
		os.write("</comment>\n");
	    }
	    
	    int score = entry.getScore();
	    Integer i_score = new Integer(score);
	    int correct = entry.getCorrect();
	    int incorrect = entry.getIncorrect();
	    int contiguous = entry.getContiguousCorrect();
	    int penalty = entry.getPenaltyScore();

	    buf.append("<score correct=\"");
	    buf.append(correct);
	    buf.append("\" ");

	    buf.append("incorrect=\"");
	    buf.append(incorrect);
	    buf.append("\" ");

	    buf.append("contiguous-correct=\"");
	    buf.append(contiguous);
	    buf.append("\" ");
	    
	    buf.append("penalty=\"");
	    buf.append(penalty);
	    buf.append("\">");
	    os.write(buf.toString());
	    buf.delete(0, buf.length());

	    os.write(i_score.toString());
	    os.write("</score>\n");
	    os.write("</entry>\n");
	}
    }

    private DateFormat getDateFormat()
    {
	DateFormat df = new SimpleDateFormat("dd/MM/yy hh:mm:ss");
	return df;
    }

    private void saveSessions(Writer os) throws IOException
    {
	os.write("\n<sessions>\n\n");
	ArrayList sessions = getSessions();
	Iterator iterator = sessions.iterator();

	DateFormat df = getDateFormat();
	
	while (iterator.hasNext()) {
	    SessionEntry entry = (SessionEntry) iterator.next();
	    Date date = entry.getDate();
	    int score = entry.getScore();

	    String formattedDate = df.format(date);
	    os.write("<session-entry date=\"" + formattedDate + "\">\n");

	    os.write("<session-score ");
	    for (int index = 0; index < states.length; index++) {
		int value = entry.getSessionScore(states[index]);
		os.write(" " + series[index] + "=\"" + value + "\"");
	    }
	    os.write(">");
	    os.write(score + "\n");
	    os.write("</session-score>\n");
	    os.write("</session-entry>\n\n");
	}
	os.write("</sessions>\n");
    }

    public void save(Writer os) throws IOException
    {
	os.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
	os.write("<wordlist name=\"" + getName() + "\" lang=\"fr\" " +
		 "trans=\"en\">\n");

	saveEntries(os);
	if (sessionList != null && sessionList.size() > 0) {
	    saveSessions(os);
	}

	os.write("</wordlist>\n");
	os.flush();
	cardList.setModified(false);
    }

    private void processXML(Document doc)
    {
	Element rootElement = doc.getDocumentElement();
	String name = rootElement.getAttribute("name");
	cardList.setName(name);
	
	NodeList nlist = doc.getElementsByTagName("entry");
	for (int index = 0; index < nlist.getLength(); index++) {
	    Node node = nlist.item(index);
	    
	    Entry entry = new Entry();
	    assignEntryValues(node, entry);
	    
	    cardList.add(entry);
	}

	nlist = doc.getElementsByTagName("session-entry");
	for (int index = 0; index < nlist.getLength(); index++) {
	    Node node = nlist.item(index);
	    
	    SessionEntry entry = new SessionEntry();
	    assignSessionValues(node, entry);
	    sessionList.add(entry);
	}

	// Add this session
	currentSession = new SessionEntry();
	currentSession.setSessionScore(cardList);
	sessionList.add(currentSession);
    }

    private void assignEntryValues(Node node, Entry entry)
    {
	// Now get the word and translation, etc.
	NodeList children = node.getChildNodes();
	for (int cindex = 0; cindex < children.getLength(); cindex++) {
	    Node child = children.item(cindex);
	    String parameter = child.getNodeName();
	    NamedNodeMap attrlist = child.getAttributes();
	    
	    // Now process each parameter
	    NodeList values = child.getChildNodes();
	    for (int vindex = 0; vindex < values.getLength(); vindex++) {
		Node valueNode = values.item(vindex);
		String value = valueNode.getNodeValue();

		assignValue(entry, parameter, value, attrlist);
	    }
	}
    }

    private void assignValue(Entry entry, String parameter, String value,
			     NamedNodeMap attrlist)
    {
	if (parameter.equals("word")) {
	    entry.setWord(value);
	    if (attrlist != null) {
		Node prefixNode = attrlist.getNamedItem("prefix");
		if (prefixNode != null) {
		    entry.setPrefix(prefixNode.getNodeValue());
		}
	    }
	} else if (parameter.equals("trans")) {
	    entry.setTranslation(value);

	    if (attrlist != null) {
		Node transPrefixNode = attrlist.getNamedItem("prefix");
		if (transPrefixNode != null) {
		    entry.setTranslationPrefix(transPrefixNode.getNodeValue());
		}
	    }
	} else if (parameter.equals("comment")) {
	    entry.setComment(value);
	    
	} else if (parameter.equals("score")) {
	    int score = Integer.parseInt(value);
	    entry.setScore(score);
	    
	    Node scoreIncorrect = attrlist.getNamedItem("incorrect");
	    if (scoreIncorrect != null) {
		int incorrect = Integer.parseInt(scoreIncorrect.getNodeValue());
		entry.setIncorrect(incorrect);
	    }

	    Node scoreCorrect = attrlist.getNamedItem("correct");
	    if (scoreCorrect != null) {
		int correct = Integer.parseInt(scoreCorrect.getNodeValue());
		entry.setCorrect(correct);
	    }

	    Node inArow = attrlist.getNamedItem("contiguous-correct");
	    if (inArow != null) {
		int contiguous_correct = 
		    Integer.parseInt(inArow.getNodeValue());
		entry.setContiguousCorrect(contiguous_correct);
	    }

	    Node penaltyNode = attrlist.getNamedItem("penalty");
	    if (penaltyNode != null) {
		int penalty = Integer.parseInt(penaltyNode.getNodeValue());
		entry.setPenaltyScore(penalty);
	    }
		
	} 
    }

    private void assignSessionValues(Node node, SessionEntry entry)
    {
	NodeList children = node.getChildNodes();
	NamedNodeMap nodeAttrList = node.getAttributes();
	if (nodeAttrList != null) {
	    Node dateNode = nodeAttrList.getNamedItem("date");
	    if (dateNode != null) {
		String dateValue = dateNode.getNodeValue();
		DateFormat df = getDateFormat();
		try {
		    Date date = df.parse(dateValue);
		    entry.setDate(date);
		} catch (ParseException e) {
		    System.err.println("Unable to set date: " + dateValue);
		}
	    }
	}
	for (int cindex = 0; cindex < children.getLength(); cindex++) {
	    Node child = children.item(cindex);
	    String parameter = child.getNodeName();
	    NamedNodeMap attrlist = child.getAttributes();
	    
	    // Now process each parameter
	    NodeList values = child.getChildNodes();
	    for (int vindex = 0; vindex < values.getLength(); vindex++) {
		Node valueNode = values.item(vindex);
		String value = valueNode.getNodeValue();
		value = value.trim();
		assignSessionValue(entry, parameter, value, attrlist);
	    }
	}
    }

    private void assignSessionValue(SessionEntry entry, String parameter,
				    String value, NamedNodeMap attrlist)
    {
	if (parameter.equals("session-score")) {
	    entry.setScore(Integer.parseInt(value));
	    if (attrlist != null) {
		String svalue = null;

		for (int index = 0; index < states.length; index++) {
		    Node node = attrlist.getNamedItem(series[index]);
		    if (node != null) {
			svalue = node.getNodeValue();
			if (svalue != null) {
			    svalue = svalue.trim();
			    Integer score = new Integer(svalue);
			    entry.setSessionScore(states[index], 
						  score.intValue());
			}
		    }
		}
	    }
	}
    }

    private boolean repeatState(int currentState)
    {
	boolean repeated = false;
	int count = 0;

	switch (currentState) {
	    
	case State.EXCELLENT:
	    count += cardList.size(State.AVERAGE);
	case State.GOOD:
	    count += cardList.size(State.GOOD);
	case State.AVERAGE:
	    count += cardList.size(State.POOR);
	    break;
	}
	if (count >= 5) {
	    repeated = true;
	}
	return repeated;
    }
    
    private int findMaxState()
    {
	int[] states = {State.POOR, State.AVERAGE, State.GOOD,
			State.EXCELLENT};
	int maxState = -1;
	int i = 0;
	while (i < states.length) {
	    if (cardList.isEnabled(states[i])) {
		maxState = states[i];
	    }
	    i++;
	}
	return maxState;
    }

    public Entry getNextEntryFromCircularList(int type)
    {
	if (itr != null) {
	    if (itr.hasNext()) {
		currentEntry = (Entry) itr.next();
	    } else {
		currentEntry = null;
	    }
	}
	return currentEntry;
    }

    public CardList getCardList()
    {
	return cardList;
    }

    private String getName()
    {
	return cardList.getName();
    }

    public Entry getCurrentEntry()
    {
	return currentEntry;
    }

    public void add(Entry entry)
    {
	cardList.add(entry);
	if (currentEntry == null) {
	    currentEntry = entry;
	}
	fireXCardEvent("ENGINE_ADD_ENTRY", this);
    }

    public ArrayList getSessions()
    {
	if (currentSession != null) {
	    currentSession.setSessionScore(cardList);
	}
	return sessionList;
    }

    void clearSessions()
    {
	sessionList = null;
    }

    public boolean isCardListModified()
    {
	return cardList.isModified();
    }

    public void removeAllEntries()
    {
	cardList.clear();
    }

    public void repositionEntry()
    {
	if (itr == null) {
	    itr = cardList.iterator();
	    itr.setCircular(true);
	}
	itr.reposition();
    }

    public int getNumVisitedEntries()
    {
	return cardList.getNumVisitedEntries();
    }

    public void resetIterator()
    {
	itr.reset();
    }

    // XXX Currently used by XCardEditPanel.  Need to think of a new
    // mechanism
    public void setCardListModified(boolean value)
    {
	cardList.setModified(value);
    }

    public static void main(String[] argv)
    {
	try {
	    CardEngine engine = new CardEngine();
	    Class c = engine.getClass();
	    ClassLoader loader = c.getClassLoader();
	    //URL wordList = loader.getResource("words/clothing.xml");
	    URL wordList = new URL("file:" + argv[0]);
	    System.out.println(wordList.toString());
	    engine.load(wordList.openStream());

	    CardList clist = engine.getCardList();
	    clist.randomize();
	    clist.sortByScore();

	    int totalAverage = 
		clist.size(State.AVERAGE);
	    System.out.println("Total average: " + totalAverage);
	    CardList list = engine.getCardList();
	    Iterator iterator = list.iterator();
	    while (iterator.hasNext()) {
		Entry entry = (Entry) iterator.next();
		System.out.println("word: " + entry.getFullWord() + " " +
				   "translation: " + 
				   entry.getFullTranslation());
		System.out.print("score: " + entry.getScore() + " ");
		System.out.print("correct: " + entry.getCorrect() + " ");
		System.out.println("incorrect: " + entry.getIncorrect());
		System.out.println("Correct in a row: " + 
				   entry.getContiguousCorrect());
		System.out.println("comment: " + entry.getComment());
	    }

	    //OutputStreamWriter os = new OutputStreamWriter(System.out);
	    //engine.save(os);

	    /*
	      ArrayList sessions = engine.getSessions();
	    SessionEntry[] sentries = (SessionEntry[])
		sessions.toArray(new SessionEntry[0]);
	    for (int sindex = 0; sindex < sentries.length; sindex++) {
		System.out.println(sentries[sindex].getDate());
	    }
	    */

	} catch(SAXParseException error) {
	    System.err.println("Parring error line: " + 
			       error.getLineNumber());
	    System.err.println(error.getMessage());
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

}
