/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /h/ellis/source/java/xcard/RCS/CardList.java,v 1.12 2001/09/28 04:04:27 ellis Exp $
 */

package xcard;

import java.util.*;

public class CardList  {
    
    ArrayList excellentList = new ArrayList();
    ArrayList goodList = new ArrayList();
    ArrayList avgList = new ArrayList();
    ArrayList poorList = new ArrayList();
    
    ArrayList[] lists = {excellentList, goodList, avgList, poorList};
    ArrayList[] allLists = {excellentList, goodList, avgList, poorList};
    
    String name = null;
    private boolean dataModified = false;
    VisitedEntryListener ve = new VisitedEntryListener();

    public void setName(String name)
    {
	this.name = name;
    }

    public String getName()
    {
	return name;
    }

    private ArrayList getListByType(int listType)
    {
	ArrayList list = null;

	switch (listType) {
	    
	case State.EXCELLENT:
	    list = excellentList;
	    break;

	case State.GOOD:
	    list = goodList;
	    break;
	    
	case State.AVERAGE:
	    list = avgList;
	    break;

	case State.POOR:
	    list = poorList;
	    break;
	}
	return list;
    }

    public void enable(int listType)
    {
	lists[listType] = getListByType(listType);
    }

    public void disable(int listType)
    {
	lists[listType] = null;
    }

    public boolean isEnabled(int listType)
    {
	boolean result = false;
	if (listType < lists.length && listType >= 0) {
	    if (lists[listType] != null) {
		result = true;
	    }
	}
	return result;
    }

    public void computeEntryScores()
    {
	for (int i = 0; i < lists.length; i++) {
	    Iterator iterator = lists[i].iterator();
	    while (iterator.hasNext()) {
		Entry entry = (Entry) iterator.next();
		entry.computeScore();
	    }
	}
    }

    public void add(Entry entry)
    {
	ArrayList list = getListByType(entry.getState());
	entry.setXCardEventListener(ve);
	list.add(entry);
    }

    public void sortedAdd(Entry entry)
    {
	ArrayList list = getListByType(entry.getState());
 	int score = entry.getScore();
	int index = 0;
	int size = list.size();
	boolean addedEntry = false;

	while (index < size) {
	    Entry e = (Entry) list.get(index);
 	    int entryScore = e.getScore();
 	    if (entryScore > score) {
		entry.setXCardEventListener(ve);
		
		log("Added at " + index);
		
 		list.add(index, entry);
 		addedEntry = true;
 		break;
 	    }
 	    index++;
 	}

	// Defensive measure incase the list is not sorted which is 
	// entirely possible depending on the user inputs
 	if (! addedEntry) {
 	    list.add(entry);
	    log("Added at the end");
 	}
    }

    private void printList(ArrayList list)
    {
	if (list == poorList) {
	    System.out.println("Poor List");
	} else if (list == avgList) {
	    System.out.println("AVG List");
	}  else if (list == goodList) {
	    System.out.println("Good List");
	}  else if (list == excellentList) {
	    System.out.println("Excellent List");
	} else {
	    System.out.println("Don't know what this is " + list);
	}
    }

    public Entry get(Entry entry)
    {
	Entry result = null;
	int index = -1;
	for (int i = 0; i < lists.length; i++) {
	    if (lists[i] != null) {
		index = lists[i].indexOf(entry);
		if (index != -1) {
		    result = (Entry) lists[i].get(index);
		    break;
		}
	    }
	}
	return result;
    }

    public void remove(Entry entry)
    {
	Iterator iterator = iterator();
	while (iterator.hasNext()) {
	    Entry e = (Entry) iterator.next();
	    if (e.equals(entry)) {
		iterator.remove();
		break;
	    }
	}
    }

    public int size()
    {
	int size = 0;
	for (int i = 0; i < allLists.length; i++) {
	    size += allLists[i].size();
	}
	return size;
    }

    public int size(int listType)
    {
	return allLists[listType].size();
    }


    public void clear()
    {
	for (int i = 0; i < allLists.length; i++) {
	    if (allLists[i] != null) {
		allLists[i].clear();
	    }
	}
	name = null;
	dataModified = false;
	ve.setNumVisited(0);
    }

    public void addAll(List list)
    {
	Iterator iterator = list.iterator();
	Entry e = null;
	while (iterator.hasNext()) {
	    e = (Entry) iterator.next();
	    add(e);
	}
    }

    public CardListIterator iterator()
    {
	return new CardListItr();
    }

    private int getListState(ArrayList list)
    {
	int state = State.POOR;;
	if (list == poorList) {
	    state = State.POOR;
	} else if (list == avgList) {
	    state = State.AVERAGE;
	}  else if (list == goodList) {
	    state = State.GOOD;
	}  else if (list == excellentList) {
	    state = State.EXCELLENT;
	}
	return state;
    }


    private class CardListItr implements CardListIterator {
	boolean circular = false;
	boolean allVisible = true;
	int currentListIndex = 0;
	ArrayList currentList = null;
	Entry currentEntry = null;
	int priorEntryIncorrect = 0;

	public void setAllStatesVisible(boolean value)
	{
	    allVisible = value;
	}

	public boolean isAllStatesVisible()
	{
	    return allVisible;
	}

 	private ArrayList[] getAvailableLists()
 	{
 	    ArrayList[] result = allLists;
 	    if (! allVisible) {
 		result = lists;
 	    }
 	    return result;
 	}


	private boolean isAlistAvailable()
	{
	    boolean result = false;
	    for (int i = 0; i < lists.length; i++) {
		if (lists[i] != null && lists[i].size() > 0) {
		    result = true;
		    break;
		}
	    }
	    return result;
	}

	private ArrayList getNextList(ArrayList list)
	{
	    ArrayList foundList = null, tmp = null;
	    ArrayList[] lists = getAvailableLists();

	    for (int i = lists.length - 1; i >= 0; i--) {
		tmp = lists[i];
		if (tmp != null && tmp.size() > 0) {
		    if (tmp == list) {
			i--;
			for (int y = i; y >= 0; y--) {
			    if (lists[y] != null && lists[y].size() > 0) {
				foundList = lists[y];
				break;
			    }
			}
		    } else {
			// Passing in a null will return the first 
			// non-null list
			if (list == null) {
			    foundList = tmp;
			    break;
			}
		    }
		}
	    }

	    // Wrap around if possible
	    if (foundList == null && circular && isAlistAvailable()) {
		log("Wrapping");
		foundList = getNextList(null);
	    }

	    return foundList;
	}

	public boolean hasNext() 
	{
	    boolean hasNext = false;

	    // Init currentList
	    if (currentList == null || currentList.size() == 0) {
		currentList = getNextList(currentList);
		log("Trying to get new list");

	    } else {
		// Now check to see if the currentList is enabled.  Someone
		// might have turned it off.  Only check if we are not in 
		// allVisible mode
		if (! allVisible) {
		    int state = getListState(currentList);
		    if (! isEnabled(state) ) {
			currentList = getNextList(currentList);
		    }
		}
	    }

	    if (currentList != null) {
		log("CurrentListIndex: " + currentListIndex + 
		    " ListSize: " +  currentList.size());

		if (currentListIndex < currentList.size()) {
		    hasNext = true;
		} else {
		    // Get the next list
		    currentList = getNextList(currentList);
		    if (currentList != null) {
			currentListIndex = 0;
			if (currentListIndex < currentList.size()) {
			    hasNext = true;
			}
		    }
		}
	    }
	    log("hasNext: " + hasNext);
	    return hasNext;
	}

	public Object next()
	{
	    log("Index: " + currentListIndex);
	    
	    currentEntry = (Entry) currentList.get(currentListIndex++);
	    if (currentEntry != null) {
		priorEntryIncorrect = currentEntry.getIncorrect();
		log("Entry: " + currentEntry.getScore());
	    } else {
		priorEntryIncorrect = 0;
	    }
	    return currentEntry;
	}

	public boolean isCircular()
	{
	    return circular;
	}

	public void reset()
	{
	    currentList = null;
	    currentListIndex = 0;
	    currentEntry = null;
	    priorEntryIncorrect = 0;
	}

	public void setCircular(boolean circular)
	{
	    this.circular = circular;
	}

	public void remove()
	{
	    currentList.remove(currentEntry);
	}

	public void reposition()
	{
	    log("Reposition");

	    if (currentList != null) {
		if (currentEntry != null) {
		    int state = currentEntry.getState();
		    int listState = getListState(currentList);

		    // We have a transistion
		    if (state != listState) {
			dataModified = true;
			log("\tState Transistion");

			// XXX Doing a remove from two lists.  This is needed
			// because reposition may be called more than once
			// We could search the lists and try finding the
			// correct list or we just call delete
			if (!currentList.remove(currentEntry)) {
			    ArrayList list = getListByType(state);
			    list.remove(currentEntry);
			}
			
			sortedAdd(currentEntry);

			currentListIndex--;
			if (currentListIndex < 0) {
			    currentListIndex = 0;
			}
		    } else {
			// reposition if incorrect has changed
			int currentIncorrect = currentEntry.getIncorrect();
			if (currentIncorrect > priorEntryIncorrect) {
			    if (currentList.size() > 1) {
				dataModified = true;
				log("Moving: " + currentIncorrect  +
				    " < " + priorEntryIncorrect);
				
				currentList.remove(currentEntry);
 				currentList.add((currentListIndex + 3) % 
 						currentList.size(), 
						currentEntry);
 			    } 
 			} else {
			    currentList.remove(currentEntry);
			    sortedAdd(currentEntry);
			    currentListIndex--;
			    if (currentListIndex < 0) {
				currentListIndex = 0;
			    } 
			}
		    }

		    if (currentListIndex == currentList.size()) {
			log("Resetting currentListIndex");
			currentListIndex = 0;
		    }
		}
	    }
	}
    }


    public void randomize()
    {
	for (int i = 0; i < lists.length; i++) {
	    if (lists[i] != null) {
		ArrayList sourceList = (ArrayList) lists[i].clone();
		ArrayList newList = new ArrayList();
		Random rand = new Random();

		while (sourceList.size() > 0) {
		    int random = rand.nextInt(sourceList.size());
		    Entry entry = (Entry) sourceList.get(random);
		    newList.add(entry);
		    sourceList.remove(random);
		}
		ArrayList list = getListByType(i);
		list = newList;
	    }
	}
    }

    public void sort(Comparator c)
    {
	for (int i = 0; i < lists.length; i++) {
	    if(lists[i] != null) {
		Object[] array = lists[i].toArray();
		Arrays.sort(array, c);
		List list = Arrays.asList(array);
		lists[i].clear();
		lists[i].addAll(list);
	    }
	}
    }


    public void sortByScore()
    {
	Comparator c = new EntryScoreComparator();
	sort(c);
    }

    public void sortByName()
    {
	Comparator c = new EntryNameComparator();
	sort(c);
    }

    public Entry[] toSortedArrayByName()
    {
	Entry[] entries = new Entry[size()];

	int index = 0;
	Iterator itr = iterator();
	Entry e = null;
	while (itr.hasNext()) {
	    e = (Entry) itr.next();
	    if (e != null) {
		entries[index++] = e;
	    } else {
		System.err.println("We have a null in the list");
	    }
	}
	Arrays.sort(entries);
	return entries;
    }

    public int getNumVisitedEntries()
    {
	return ve.getNumVisited();
    }

    public boolean isModified()
    {
	return dataModified;
    }

    public void setModified(boolean value)
    {
	dataModified = value;
    }

    public Entry get(String fullWord)
    {
	Entry result = null;

	Iterator iterator = iterator();
	while (iterator.hasNext()) {
	    Entry entry = (Entry) iterator.next();
	    String entryFullWord = entry.getFullWord();
	    if (entryFullWord.equals(fullWord)) {
		result = entry;
		break;
	    }
	}
	return result;
    }


    private void log(String msg)
    {
	boolean debug = false;
	
	if (debug) {
	    System.out.println(msg);
	}
    }
}
