/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /h/ellis/source/java/xcard/RCS/Entry.java,v 1.14 2001/09/28 04:12:27 ellis Exp $
 */

package xcard;

import java.awt.event.*;

public class Entry implements Comparable {
    String prefix = null;
    String word = null;

    String translationPrefix = null;
    String translation = null;
    String comment = null;
    
    Integer score = new Integer(0);
    Integer correct = new Integer(0);
    Integer incorrect = new Integer(0);
    
    Integer contiguous_correct = new Integer(0);
    Integer penaltyScore = new Integer(0);

    boolean visited = false;

    XCardEventListener xl = null;

    public void set(Entry entry)
    {
	prefix = entry.getPrefix();
	word = entry.getWord();
	translationPrefix = entry.getTranslationPrefix();
	translation = entry.getTranslation();
	comment = entry.getComment();

	score = new Integer(entry.getScore());
	correct = new Integer(entry.getCorrect());
	incorrect = new Integer(entry.getIncorrect());
	contiguous_correct = new Integer(entry.getContiguousCorrect());
	penaltyScore = new Integer(entry.getPenaltyScore());
	if (entry.beenVisited()) {
	    setVisited(true);
	} else {
	    setVisited(false);
	}
    }

    public String getPrefix()
    {
	return prefix;
    }
     
    public void setPrefix(String prefix)
    {
	this.prefix = prefix;
    }

    public String getWord()
    {
	return word;
    }

    public String getFullWord()
    {
	String full = word;

	if (prefix != null && prefix.length() > 0) {
	    full = new String(prefix + " " + word);
	}
	return full;
    }

    public void setWord(String word)
    {
	this.word = word;
    }

    public void setTranslationPrefix(String prefix)
    {
	translationPrefix = prefix;
    }

    public String getTranslationPrefix()
    {
	return translationPrefix;
    }

    public String getTranslation()
    {
	return translation;
    }
    
    public String getFullTranslation()
    {
	String full = translation;
	if (translationPrefix != null && translationPrefix.length() > 0) {
	    full = new String(translationPrefix + " " + full);
	}
	return full;
    }


    public void setTranslation(String trans)
    {
	translation = trans;
    }

    public void setComment(String comment)
    {
	this.comment = comment;
    }

    public String getComment()
    {
	return comment;
    }

    public void setScore(int score)
    {
	this.score = new Integer(score);
    }

    public int getScore()
    {
	return score.intValue();
    }

    public void computeScore()
    {
	int corr = correct.intValue();
	int incorr = incorrect.intValue();
	int inarow = contiguous_correct.intValue();
	int penalty = penaltyScore.intValue();

	int tmp_score = ((corr + (inarow / 2)) - (incorr + penalty)) * 10;
	if (tmp_score > 90) {
	    tmp_score = 90;
	} else if (tmp_score <= -30) {
	    tmp_score = -30;
	}

	// Now convert to radians and take the sin
	double comp_score = Math.sin(tmp_score * (Math.PI / 180)) * 100;
	score = new Integer((int) Math.rint(comp_score));
    }

    public void setCorrect(int correct)
    {
	this.correct = new Integer(correct);
	if (penaltyScore.intValue() > 0) {
	    int value = penaltyScore.intValue();
	    setPenaltyScore((--value) / 2);
	}
    }

    public int getCorrect()
    {
	return correct.intValue();
    }

    public void setContiguousCorrect(int correct)
    {
	contiguous_correct = new Integer(correct);
    }

    public int getContiguousCorrect()
    {
	return contiguous_correct.intValue();
    }

    public void setPenaltyScore(int score)
    {
	penaltyScore = new Integer(score);
    }

    public int getPenaltyScore()
    {
	return penaltyScore.intValue();
    }


    public void setIncorrect(int incorrect)
    {
	this.incorrect = new Integer(incorrect);
	int state = getState();
	if (state == State.EXCELLENT) {
	    setPenaltyScore(incorrect);
	} else if (state == State.GOOD) {
	    setPenaltyScore(incorrect / 2);
	}
    }

    public int getIncorrect()
    {
	return incorrect.intValue();
    }

    public int getState()
    {
	int state = State.POOR;
	int sc = score.intValue();

	if (sc <= 0) {
	    state = State.POOR;
	} else if (sc > 0 && sc < 30) {
	    state = State.AVERAGE;
	} else if (sc >= 30 && sc < 60) {
	    state = State.GOOD;
	} else if (sc >= 60) {
	    state = State.EXCELLENT;
	}
	return state;
    }

    /*
    public String toString()
    {
	return getFullWord();
    }
    */

    public boolean equals(Entry entry)
    {
	boolean result = false;
	String fullWord = getFullWord();
	if (entry != null) {
	    if (fullWord.equals(entry.getFullWord())) {
		result = true;
	    }
	}
	return result;
    }

    public int compareTo(Object obj)
    {
	int return_value;

	if (! (obj instanceof Entry)) {
	    throw new ClassCastException();
	}
	
	Entry e = (Entry) obj;
	String word = e.getWord();
	String thisWord = getWord();

	return_value = thisWord.compareTo(word);
	
	return return_value;
    }

    public boolean beenVisited()
    {
	return visited;
    }

    public void setVisited(boolean visited)
    {
	this.visited = visited;
	fireActionEvent("ENTRY_VISITED");
    }
    
    public void incrementIncorrect()
    {
	int incorr = getIncorrect();
	setIncorrect(++incorr);
	setContiguousCorrect(0);
	computeScore();
	fireActionEvent("ENTRY_INCORRECT");
    }

    public void incrementCorrect()
    {
	int corr = getCorrect();
	setCorrect(++corr);

	int inarow = getContiguousCorrect();
	setContiguousCorrect(++inarow);
	computeScore();
	fireActionEvent("ENTRY_CORRECT");
    }
    
    public void setXCardEventListener(XCardEventListener xl)
    {
	this.xl = xl;
    }

    private void fireActionEvent(String action)
    {
	XCardEvent event = new XCardEvent(this);
	event.setName(action);

	if (xl != null) {
	    xl.actionEvent(event);
	}
    }
}
