/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/Lookup.java,v 1.9 2003/12/08 00:06:15 ellis Exp $
 */

package xcard;

import javax.swing.*;
import java.net.*;
import java.io.*;
import java.util.*;

public class Lookup extends XCardBase implements Runnable {
    private String message = null;
    private String text = null;
    private String translatedText = null;
    private String inputLanguage = "fr";

    protected void setStatus(String msg)
    {
	message = msg;
	fireXCardEvent("LOOKUP_STATUS", message);
    }

    public String getStatus()
    {
	return message;
    }
    
    public String getTranslatedText()
    {
	return translatedText;
    }

    public void setTextToTranslate(String text)
    {
	this.text = text;
    }

    public String getTextToTranslate()
    {
	return text;
    }

    public void setInputLanguage(String lang)
    {
	inputLanguage = lang;
    }

    public String getInputLanguage()
    {
	return inputLanguage;
    }

    public void run()
    {
	try {
	    getTranslation(text);
	} catch (IOException e) {
	    setStatus(e.getMessage());
	}
    }

    public String getTranslation(String text) throws IOException
    {
	URL url = new URL("http://babelfish.altavista.com/babelfish/tr");
	HttpURLConnection httpConn = (HttpURLConnection) url.openConnection();
	httpConn.setRequestMethod("POST");
	httpConn.setDoOutput(true);

	setStatus("Trying to connect");
	httpConn.connect();

	setStatus("Making request");
	OutputStream os = httpConn.getOutputStream();
	OutputStreamWriter ow = new OutputStreamWriter(os);

	String request = URLEncoder.encode(text, "UTF-8");
	
	String translateDirection = null;
	if (inputLanguage.equals("en")) {
	    translateDirection = "en_fr";
	} else {
	    translateDirection = "fr_en";
	}

	ow.write("&ienc=utf8&kls=O&doit=done&tt=urltext&intl=1&lp="+translateDirection+"&urltext=" + request);
	ow.close();
	os.close();

	setStatus("Reading response");
	InputStream is = httpConn.getInputStream();

	String data = null;
	StringBuffer dataBuffer = new StringBuffer();
	Resource rs = new Resource();

	byte[] b = new byte[1024];
	while (is.read(b, 0, b.length) >= 0) {
	    dataBuffer.append(new String(b, "ISO-8859-1"));
	    //dataBuffer.append(new String(b, "UTF-8"));
	}
	is.close();
	data = dataBuffer.toString();
	
	String translation = null;
	String textArea = "name=\"q\" value=\"";
	
	StringTokenizer str = new StringTokenizer(data, "\n");
	
	setStatus("Parsing response");

	while (str.hasMoreTokens()) {
	    String line = str.nextToken();
	    int index = line.indexOf(textArea);
	    if (index >= 0) {
		index += textArea.length();
		int endIndex = line.indexOf("\"", index);
		if (endIndex >= 0) {
		    String value = line.substring(index, endIndex);
		    translation = value.trim();
		    break;
		}
	    }
	}
	setStatus("Finished");
	translatedText = translation;
	fireXCardEvent("LOOKUP_FINISHED", translatedText);
	return translation;
    }

    public static void main(String[] argv)
    {
	try {
	    Lookup l = new Lookup();
	    String trans = l.getTranslation(argv[0]);
	    System.out.println(trans);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }


}
