/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/Message.java,v 1.8 2003/12/08 00:06:15 ellis Exp $
 */

package xcard;

import java.util.*;
import javax.swing.*;

public class Message {

    public static final int YES = 1;
    public static final int NO = 2;
    public static final int CANCEL = 3;

    public int showUnSavedMessage(CardEngine engine)
    {
	int result = NO;

	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	

	String[] message = {resources.getString("UNSAVED_CHANGES") + " (" +
			    engine.getCardList().getName() + ")",
			    "",
			    resources.getString("WANT_TO_SAVE"),
			    ""};
	String title = "Unsaved changes";

	int option = 
	    JOptionPane.showConfirmDialog(null, message, title,
					  JOptionPane.YES_NO_OPTION,
					  JOptionPane.WARNING_MESSAGE);
	if (option == JOptionPane.YES_OPTION) {
	    SaveService saveService = new SaveService();
	    if (saveService.save()) {
		result = YES;
	    } else {
		result = CANCEL;
	    }
	} 
	return result;
    }

    public String showMissingListNameMessage(CardEngine engine)
    {
	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	
	String listName = 
	    JOptionPane.showInputDialog(resources.getString("LIST_NAME"));
	
	return listName;
    }


    public static void message(String key, String value, boolean error)
    {
	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	String msg = resources.getString(key);
	if (msg == null) {
	    msg = key;
	} else {
	    if (value != null) {
		int index = msg.indexOf("$1");
		if (index >= 0) {
		    String pre = msg.substring(0, index);
		    String post = msg.substring(index + 2, msg.length());
		    msg = new String(pre + value + post);
		}
	    }
	}
	
	if (error) {
	    JOptionPane.showMessageDialog(null, msg,
					  "error",
					  JOptionPane.ERROR_MESSAGE);
	} else {
	    JOptionPane.showMessageDialog(null, msg,
					  "warning",
					  JOptionPane.WARNING_MESSAGE);
	}
	
    }

    public static void errorMessage(String key,  String value)
    {
	message(key, value, true);
    }

    public static void warningMessage(String key, String value)
    {
	message(key, value, false);
    }

    public static void errorMessage(String key)
    {
	errorMessage(key, null);
    }

    public static void warningMessage(String key)
    {
	warningMessage(key, null);
    }

    public static void errorMessage(Exception e)
    {
	errorMessage(e.getMessage());
	e.printStackTrace();
    }


}
