/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/SaveService.java,v 1.6 2002/11/21 04:40:40 ellis Exp ellis $
 */

package xcard;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import javax.swing.*;

public class SaveService extends FileService implements ActionListener {

    public SaveService()
    {
	super();
	JFileChooser chooser = getFileChooser();
	if (chooser != null) {
	    chooser.addPropertyChangeListener(new SaveFileListener());
	}
    }

    private boolean normalSave(String name)
    {
	boolean result = false;
	try {
	    JFileChooser chooser = getFileChooser();
	    PreferencesService ps = XCard.getPreferencesService();
	    File udir = ps.getDefaultFileDirectory();
	    if (udir != null) {
		chooser.setCurrentDirectory(udir);
	    }

	    File dir = chooser.getCurrentDirectory();
	    File file = new File(dir.toString() + File.separatorChar +
				 name + ".xml");
	    chooser.setSelectedFile(file);
	    String message = "Please choose a place to save " +
		name + ".xml";
	    chooser.setDialogTitle(message);
	    int state = chooser.showSaveDialog(null);
	    File chosenFile = chooser.getSelectedFile();
	    
	    if (state == JFileChooser.APPROVE_OPTION && 
		chosenFile != null) {
		// Remember where the directory is
		File parent = chosenFile;
		if (! chosenFile.isDirectory()) {
		    parent = chosenFile.getParentFile();
		}
		if (parent != null && parent.isDirectory()) {
		    ps.setDefaultFileDirectory(parent.getCanonicalPath());
		}

		OutputStreamWriter os = 
		    new OutputStreamWriter(new FileOutputStream(chosenFile), 
					   "UTF-8");

		CardEngine engine = XCard.getEngineInstance();
		engine.save(os);

		os.close();
		result = true;
	    } 
	} catch (IOException e) {
	    Message.errorMessage(e);
	}

	return result;
    }

    public boolean save()
    {
	boolean result;
	CardEngine engine = XCard.getEngineInstance();
	String name = engine.getCardList().getName();
	Resource resource = getResource();
	result = normalSave(name);
	
	return result;
    }

    private class SaveFileListener implements PropertyChangeListener {

	public void propertyChange(PropertyChangeEvent e) {
	    String propName = e.getPropertyName();
	    if (propName.equals(JFileChooser.SELECTED_FILE_CHANGED_PROPERTY)) {
		if (chooser.getDialogType() == JFileChooser.SAVE_DIALOG) {
				    
		    File selected = chooser.getSelectedFile();
		    if (selected != null && selected.isDirectory()) {
			CardEngine engine = XCard.getEngineInstance();
			String name = engine.getCardList().getName();
			File dir = chooser.getCurrentDirectory();
			File cfile = 
			    new File(dir.toString() + File.separatorChar +
				     name + ".xml");
			chooser.setSelectedFile(cfile);
		    }
		}
	    }
	}
    }
    
    public void actionPerformed(ActionEvent e)
    {
	save();
    }
}
