/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/SoundMaster.java,v 1.7 2001/12/11 06:06:33 ellis Exp $
 */

package xcard;

import java.io.IOException;
import java.io.File;
import java.net.URL;
import javax.sound.midi.*;
import javax.sound.sampled.*;


public class SoundMaster implements Runnable {

    private int CORRECT = 0;
    private int INCORRECT = 1;
    
    private int volume = 6;
    AudioInputStream correctStream = null;
    Clip correctClip = null;

    AudioInputStream incorrectStream = null;
    Clip incorrectClip = null;
    
    AudioInputStream fileStream = null;
    Clip fileClip = null;

    SoundThread soundThread = new SoundThread();

    boolean enabled = true;

    public SoundMaster() {
	soundThread.start();
    }

    public void open()
    {
	Resource resource = new Resource();
	URL correct = resource.get("xcard/resources/corkpop2.wav");
	URL incorrect = resource.get("xcard/resources/guitarboing.wav");

	try {
	    correctStream = AudioSystem.getAudioInputStream(correct);
	    correctClip = getClip(correct, correctStream);

	    incorrectStream = AudioSystem.getAudioInputStream(incorrect);
	    incorrectClip = getClip(incorrect, incorrectStream);
	} catch (UnsupportedAudioFileException e) {
	    e.printStackTrace();
	} catch (IOException e) {
	    e.printStackTrace();
	}
    }

    public void close()
    {
	if (correctClip != null) {
	    correctClip.close();
	}
	if (incorrectClip != null) {
	    incorrectClip.close();
	}
    }

    public void run()
    {
	open();
    }

    public void playCorrect()
    {
	soundThread.playSound(correctClip, correctStream);
    }

    public void playIncorrect()
    {
	soundThread.playSound(incorrectClip, incorrectStream);
    }

    public void playFile(File file)
    {
	try {
	    fileStream = AudioSystem.getAudioInputStream(file);
	    fileClip = getClip(file.toURL(), fileStream);
	    soundThread.playSound(fileClip, fileStream);
	} catch (UnsupportedAudioFileException e) {
	    e.printStackTrace();
	} catch (IOException e) {
	    e.printStackTrace();
	}
    }


    private void playSound(Clip clip, AudioInputStream stream)
    {
	if (clip != null && stream != null) {
	    clip.start();
	    while (clip.isActive()) {
		Thread.yield();
		try {
		    Thread.sleep(99);
		} catch (Exception e) {
		    e.printStackTrace();
		    break;
		}
	    }
	    clip.stop();
	}
    }
    
    private Clip getClip(URL sound, AudioInputStream stream)
    {
	Clip clip = null;
	try {
	    if (sound != null) {
		AudioFormat format = stream.getFormat();
		DataLine.Info info = 
		    new DataLine.Info(Clip.class, format);
		clip = (Clip) AudioSystem.getLine(info);
		clip.open(stream);
		
		FloatControl gainControl = (FloatControl) 
		    clip.getControl(FloatControl.Type.MASTER_GAIN);
		float db = (float) (Math.log(volume)/Math.log(10.0)* 20.0);
		gainControl.setValue(db);
	    }
	} catch (LineUnavailableException ignore) {
	    //ignore.printStackTrace();
	} catch (IOException e) {
	    e.printStackTrace();
	}
	
	return clip;
    }

    class SoundThread extends Thread {
	Clip clip = null;
	AudioInputStream stream = null;

	public void run()
	{
	    waitForRequest();
	}

	private synchronized void waitForRequest()
	{
	    while(true) {
		try {
		    wait();
		    renderSound();
		} catch (InterruptedException e) {
		    e.printStackTrace();
		    break;
		}
	    }
	}

	public synchronized void playSound(Clip clip, AudioInputStream stream)
	{
	    if (enabled) {
		this.clip = clip;
		this.stream = stream;
		notify();
	    }
	}

	private void renderSound()
	{
	    if (clip != null && stream != null) {
		clip.start();
		while (clip.isActive()) {
		    Thread.yield();
		    try {
			Thread.sleep(99);
		    } catch (Exception e) {
			e.printStackTrace();
			break;
		    }
		}
		clip.stop();
	    }
	}
    }

    public boolean isEnabled()
    {
	return enabled;
    }

    public void enableSound()
    {
	enabled = true;
    }
	
    public void disableSound()
    {
	enabled = false;
    }

    public static void main(String[] argv)
    {
	SoundMaster master = new SoundMaster();
	master.open();
	for (int loop = 0; loop < 10; loop++) {
	    System.out.println(loop);
	    master.playCorrect();
	    master.playIncorrect();
	}
	master.close();
    }
}
