/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/WordMgmt.java,v 1.6 2002/11/21 04:40:40 ellis Exp $
 */

package xcard;

import java.io.*;
import java.util.*;

public class WordMgmt {
    
    public void stripScores(String filename, CardEngine engine)
    {
	try {
	    engine.load(new File(filename));
	    CardList list = engine.getCardList();
	    list.sortByName();

	    Iterator itr = list.iterator();
	    while (itr.hasNext()) {
		Entry entry = (Entry) itr.next();
		entry.setCorrect(0);
		entry.setIncorrect(0);
		entry.setContiguousCorrect(0);
		entry.setPenaltyScore(0);
		entry.setScore(0);
	    }
	    engine.clearSessions();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public void importFile(String filename, CardEngine engine)
    {
	try {
	    BufferedReader br = new BufferedReader(new FileReader(filename));
	    String line = null;
	    while ((line = br.readLine()) != null) {
		line = line.trim();

		int index = line.indexOf(",");
		
		Entry entry = new Entry();
		if (index >= 0) {
		    String word = line.substring(0, index);
		    word = word.trim();
		    String translation = line.substring(index + 1, 
							line.length());
		    String comment = null;

		    entry.setWord(word);
		    if ((index = translation.indexOf(",")) >= 0) {
			comment = translation.substring(index + 1,
							translation.length());
			comment = comment.trim();
			translation = translation.substring(0, index);
			translation = translation.trim();

			entry.setTranslation(translation);
			entry.setComment(comment);
		    } else {
			entry.setTranslation(translation);
		    }

		    engine.add(entry);

		} else {
		    if (line.length() > 0) {
			entry.setWord(line);
			engine.add(entry);
		    }
		}
	    }
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    public static void main(String[] argv)
    {
	if (argv.length < 2) {
	    System.err.println("WordMgmt {-s|-i} filename");
	    System.exit(1);
	}
	
	String option = argv[0];
	String filename = argv[1];
	try {
	    CardEngine engine = new CardEngine();
	    WordMgmt wm = new WordMgmt();
	    if (option.equals("-s")) {
		wm.stripScores(filename, engine);
	    } else if (option.equals("-i")) {
		wm.importFile(filename, engine);
	    }
	    OutputStreamWriter os = new OutputStreamWriter(System.out);
	    engine.save(os);

	} catch (Exception e) {
	    e.printStackTrace();
	}
    }
}

