/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000, 2001, 2002, 2003, 2004
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /Volumes/rage/source/java/xcard/RCS/XCard.java,v 1.40 2003/12/08 00:06:15 ellis Exp ellis $
 */

package xcard;

import java.net.URL;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.ResourceBundle;
import javax.swing.*;
import javax.swing.event.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import javax.sound.sampled.LineUnavailableException;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


public class XCard {

    public static int INITIAL_WIDTH = 510;
    public static int INITIAL_HEIGHT = 600;

    public static String[] views = new String[5];
    private String[] viewKeys = {"CARDS", "EDIT_WORDS", "PROGRESS", 
				 "TRANSLATION", "WORD_LIST"};

    private JMenuItem[] viewMenuItems = new JMenuItem[5];
    static final int CARDS = 0;
    static final int EDIT_WORDS = 1;
    static final int PROGRESS = 2;
    static final int LOOKUP = 3;
    static final int WORD_LIST = 4;

    public static final int WORD_LIST_SCREEN = 1;
    public static final int CARDS_SCREEN = 2;
 
    static CardEngine engine = null;
    static PreferencesService ps = null;
    static AudioMaster audio = null;

    private SplashScreen splash = new SplashScreen();
    
    JFrame frame = new JFrame("XCard");

    JTabbedPane tabbedPane = null;
    XCardWordListPanel wordPanel = null;
    XCardPanel cardPanel = null;
    XCardEditPanel editPanel = null;    
    XCardProgressPanel progressPanel = null;
    XCardLookupPanel lookupPanel = null;
    XCardPreferences preferences = null;
    
    SaveService saveService = null;
    OpenService openService = null;

    private static XCardEventListener engineListener = null;

    private static ResourceBundle resources = null;
    private static JMenu file = null;
    private static JMenu edit = null;
    private static JMenu windows = null;
    private static JMenu help = null;

    private class EngineListener implements XCardEventListener {
	
	public void actionEvent(XCardEvent event)
	{
	    String name = event.getName();
	    if (name.equals("ENGINE_LOAD")) {
		editPanel.updateFieldValues();
		cardPanel.updateValues();
		lookupPanel.updateValues();
	    } else {
		cardPanel.updateValues();
	    }
	}

    };

    public XCard()
    {
	UIManager.addPropertyChangeListener(new UIListener((JComponent) frame.getRootPane()));

	splash.show();

	resources = ps.getResourceBundle();
	buildViewNames(resources);
	
	saveService = new SaveService();
	openService = new OpenService();

	// Image Icon for the frame
	Resource resource = new Resource();
	URL iconURL = resource.get("xcard/resources/XCard.jpg");
	ImageIcon icon = new ImageIcon(iconURL);
	frame.setIconImage(icon.getImage());

	splash.setProgress(10);

	Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	frame.setLocation(screenSize.width/2 - INITIAL_WIDTH/2,
			  screenSize.height/2 - INITIAL_HEIGHT/2);

	splash.setProgress(20);
	JMenuBar menuBar = new JMenuBar();
	menuBar.setOpaque(true);
	file = buildFileMenu();
	windows = buildWindowMenu();
	//JMenu tools = buildToolsMenu();
	help = buildHelpMenu();

	menuBar.add(file);
	if (! Resource.isApple()) {
	    edit = buildEditMenu();
	    menuBar.add(edit);
	}
	menuBar.add(windows);
	//menuBar.add(tools);
	menuBar.add(help);

	frame.setJMenuBar(menuBar);
	
	WindowListener l = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		if (engine.isCardListModified()) {
		    Message message = new Message();
		    int saved = Message.CANCEL;
		    while (saved == Message.CANCEL) {
			saved = message.showUnSavedMessage(engine);
			if (saved == Message.YES || saved == Message.NO) {
			    exit();
			}
		    }
		} else {
		    exit();
		}
	    }
	};
	
	splash.setProgress(30);
	engine = new CardEngine();

	wordPanel = new XCardWordListPanel();
	wordPanel.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
		    String name = e.getPropertyName();
		    if (name.equals("ENGINE_LOAD")) {
			initTabbedPane();
			switchScreen(XCard.CARDS_SCREEN);
		    }
		}
	    });

	splash.setProgress(40);
	preferences = new XCardPreferences(ps);
	UIManager.addPropertyChangeListener(new UIListener((JComponent) preferences.getRootPane()));


	preferences.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
		    String name = e.getPropertyName();
		    if (name.equals("PREFERENCES_UPDATE") ||
			name.equals("PREFERENCES_TRANSDIR_CHANGE")) {
			cardPanel.updatePreferences(ps);
			
		    } else if (name.equals("PREFERENCES_LOCALE_CHANGE")) {
			cardPanel.updateLocalizedText();
			editPanel.updateLocalizedText();
			wordPanel.updateLocalizedText();
			lookupPanel.updateLocalizedText();
			progressPanel.updateLocalizedText();
			updateLocalizedText();
		    }
		}
	    });
	splash.setProgress(50);
	editPanel = new XCardEditPanel();
	editPanel.addPropertyChangeListener(new PropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent e) {
		    String name = e.getPropertyName();

		    if (name.equals("ENGINE_NEW")) {
			cardPanel.updateValues();
		    } else if (name.startsWith("ENGINE")) {
			cardPanel.updateValues();
		    }
		}
	    });

	
	splash.setProgress(60);
	cardPanel = new XCardPanel();
	
	splash.setProgress(70);
	progressPanel = new XCardProgressPanel();
	lookupPanel = new XCardLookupPanel();
	lookupPanel.addPropertyChangeListener(new PropertyChangeListener() {

		public void propertyChange(PropertyChangeEvent event)
		{
		    String name = event.getPropertyName();
		    if (name.equals("LOOKUP_ADD_ENTRY")) {
			editPanel.updateFieldValues();
		    } else if (name.equals("LOOKUP_ENGINE_NEW")) {
			cardPanel.updateValues();
			editPanel.updateFieldValues();
		    } else if (name.equals("PREFERENCES_TRANSDIR_CHANGE")) {
			cardPanel.updatePreferences(ps);
			preferences.updateTranslateTo();
		    }
		}
	    });
	
	engineListener = new EngineListener();
	engine.addXCardEventListener(engineListener);

	openService.addServiceEventListener(new ServiceEventListener() {
		
		public void actionEvent(ServiceEvent event)
		{
		    String name = event.getName();
		    if (name.equals("OPEN_SERVICE")) {
			cardPanel.updateValues();
			editPanel.updateFieldValues();

			switchScreen(CARDS_SCREEN);
			tabbedPane.setSelectedIndex(CARDS);
		    }
		}
	    });
	
	splash.setProgress(80);
	tabbedPane = new JTabbedPane();
	tabbedPane.addChangeListener(new ChangeListener() {
		public void stateChanged(ChangeEvent e) {
		    initTabbedPane();
		}
	    });
	tabbedPane.setName("tabbedPane");
	
	
	splash.setProgress(90);

	frame.addWindowListener(l);
	Container content = frame.getContentPane();
	content.add("Center", wordPanel);

	splash.setProgress(100);
	splash.dispose();

	frame.setSize(INITIAL_WIDTH, INITIAL_HEIGHT);
	frame.setVisible(true);

	if (Resource.isApple()) {
	    AppleMenu.registerMacOSXApplication(this);
	    AppleMenu.enablePrefs(true);
	}
    }

    private void exit()
    {
	frame.dispose();
	System.exit(0);
    }

    public JMenu buildFileMenu() {
	JMenu file = new JMenu(resources.getString("FILE"));
	JMenuItem open = new JMenuItem(resources.getString("OPEN"));
	open.addActionListener(openService);
	open.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,
						   Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));

	JMenuItem save = new JMenuItem(resources.getString("SAVE"));
	save.addActionListener(saveService);
	save.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,
						   Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));

	file.add(open);
	file.add(save);
	if (! Resource.isApple()) {
	    file.addSeparator();

	    JMenuItem quit = new JMenuItem(resources.getString("QUIT"));

	    quit.addActionListener(new ActionListener() {
		    public void actionPerformed(ActionEvent e) {
			quit();
		    }});
	    quit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q,
						       Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
	    file.add(quit);
	}
	
	return file;
    }

    public JMenu buildToolsMenu()
    {
	JMenu tools = new JMenu("Tools");
	JMenuItem memory = new JMenuItem("Memory");
	memory.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    Runtime rt = Runtime.getRuntime();
		    long total = rt.totalMemory();
		    long free = rt.freeMemory();
		    System.out.println("Memory Used: " + (total - free));
		    System.out.println("Total Memory: " + total);
		}});
	tools.add(memory);
	
	return tools;
    }

    public JMenu buildHelpMenu()
    {
	JMenu help = new JMenu(resources.getString("HELP"));
	JMenuItem xcardHelp = new JMenuItem(resources.getString("XCARD_HELP"));
	xcardHelp.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    try {
			BrowserService bs = new BrowserService();
			URL url = new URL("http://xcard.seul.org");
			if (! bs.displayURL(url)) {
			    Message.errorMessage("CAN'T_DISPLAY_URL", 
						 url.toString());
			}
		    } catch (Exception e1) {
			Message.errorMessage(e1);
		    }
		}
	    });

	JMenuItem about = new JMenuItem(resources.getString("ABOUT_XCARD"));
	about.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    about();
		}
	    });
	help.add(xcardHelp);
	help.add(about);
	return help;
    }

    public JMenu buildEditMenu() {
	JMenu edit = new JMenu(resources.getString("EDIT"));
	//JMenuItem copy = new JMenuItem(resources.getString("COPY"));
	//JMenuItem cut = new JMenuItem(resources.getString("CUT"));
	//JMenuItem paste = new JMenuItem(resources.getString("PASTE"));
	
	JMenuItem prefs = new JMenuItem(resources.getString("PREFERENCES"));

	prefs.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    preferences();
		}
	    });
	//	prefs.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_COMMA,
	//						    Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));


	/* Disabling copy, cut, and paste until they can really be 
	   implemented */

	//copy.setEnabled(false);
	//cut.setEnabled(false);
	//paste.setEnabled(false);

	//edit.add(cut);
	//edit.add(copy);
	//edit.add(paste);
	//edit.addSeparator();
	edit.add(prefs);
	return edit;
    }

    public JMenu buildWindowMenu() 
    {
	JMenu window = new JMenu(resources.getString("WINDOW"));

	JMenuItem minimize = new JMenuItem("Minimize");
	minimize.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    frame.setState(Frame.ICONIFIED);
		}
	    });
	minimize.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_M,
						       Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
	window.add(minimize);
	window.add(new JSeparator());

	int count = 0;
	JMenuItem cards = new JMenuItem(views[CARDS]);
	cards.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    switchScreen(CARDS_SCREEN);
		    tabbedPane.setSelectedIndex(CARDS);
		}
	    });
	viewMenuItems[count++] = cards;
	window.add(cards);

	JMenuItem edit = new JMenuItem(views[EDIT_WORDS]);
	edit.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    switchScreen(CARDS_SCREEN);
		    editPanel.updateFieldValues();
		    tabbedPane.setSelectedIndex(EDIT_WORDS);
		}
	    });
	viewMenuItems[count++] = edit;
	window.add(edit);

	JMenuItem progress = new JMenuItem(views[PROGRESS]);
	progress.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    switchScreen(CARDS_SCREEN);
		    tabbedPane.setSelectedIndex(PROGRESS);
		}
	    });
	viewMenuItems[count++] = progress;
	window.add(progress);

	JMenuItem lookup = new JMenuItem(views[LOOKUP]);
	lookup.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    switchScreen(CARDS_SCREEN);
		    tabbedPane.setSelectedIndex(LOOKUP);
		}
	    });
	lookup.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,
						     Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));

	viewMenuItems[count++] = lookup;
	window.add(lookup);


	JMenuItem word_list = new JMenuItem(views[WORD_LIST]);
	word_list.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    switchScreen(WORD_LIST_SCREEN);
		}
	    });
	word_list.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W,
							Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
	viewMenuItems[count++] = word_list;
	window.add(word_list);

	return window;
    }

    public void switchScreen(int screen)
    {
	Container container = frame.getContentPane();
	container.removeAll();

	switch (screen) {

	case WORD_LIST_SCREEN:
	    container.add(wordPanel);
	    SwingUtilities.updateComponentTreeUI(wordPanel);
	    wordPanel.revalidate();
	    wordPanel.repaint();
	    wordPanel.requestFocusInWindow();
	    break;

	case CARDS_SCREEN:
	    // XXX init the tabbed Pane
	    if (tabbedPane.getTabCount() == 0) {
		tabbedPane.add(views[CARDS], cardPanel);
		tabbedPane.add(views[EDIT_WORDS], editPanel);
		tabbedPane.add(views[PROGRESS], progressPanel);
		tabbedPane.add(views[LOOKUP], lookupPanel);
		SwingUtilities.updateComponentTreeUI(tabbedPane);
	    }
	    container.add(tabbedPane);
	    tabbedPane.revalidate();
	    container.repaint();
	    tabbedPane.requestFocusInWindow();
	    break;
	}
    }

    private void initTabbedPane()
    {
	int index = tabbedPane.getSelectedIndex();
	if (index == PROGRESS) {
	    progressPanel.updateValues();
	} else if (index == EDIT_WORDS) {
	    if(! editPanel.areFieldsComposed()) {
		editPanel.composeFields();
	    }
	} else if (index == CARDS) {
	    if (! cardPanel.areFieldsComposed()) {
		cardPanel.composeFields();
	    }
	} else if (index == LOOKUP) {
	    if (! lookupPanel.areFieldsComposed()) {
		lookupPanel.composeFields();
	    }
	}
    }

    private void updateLocalizedText()
    {
	resources = ps.getResourceBundle();// So resources gets updated
	file.setText(resources.getString("FILE"));
	if (edit != null) {
	    edit.setText(resources.getString("EDIT"));
	}
	windows.setText(resources.getString("WINDOW"));
	help.setText(resources.getString("HELP"));

	buildViewNames(resources);
	if (tabbedPane != null) {
	    for (int i = 0; i < tabbedPane.getTabCount(); i++) {
		tabbedPane.setTitleAt(i, views[i]);
	    }
	}
	if (file != null) {
	    String[] fileItems = {"OPEN", "SAVE", null, "QUIT"};
	    int count = file.getItemCount();
	    for (int i = 0; i < count; i++) {
		Component item = file.getMenuComponent(i);
		if (item != null) {
		    if (fileItems[i] != null) {
			((JMenuItem) item).setText(resources.getString(fileItems[i]));
		    }
		} 
	    }
	}
	if (windows != null) {
	    for (int i = 0; i < viewKeys.length; i++) {
		if (viewMenuItems[i] != null) {
		    viewMenuItems[i].setText(resources.getString(viewKeys[i]));
		}
	    } 
	}

	if (edit != null) {
	    //String[] editKeys = {"COPY", "CUT", "PASTE", null, "PREFERENCES"};
	    String[] editKeys = {"PREFERENCES"};
	    int count = edit.getItemCount();
	    for (int i = 0; i < count; i++) {
		Component item = edit.getMenuComponent(i);
		if (item != null && editKeys[i] != null) {
		    ((JMenuItem) item).setText(resources.getString(editKeys[i]));
		} 
	    }
	}
	    

	if (help != null) {
	    String[] helpKeys = {"XCARD_HELP", "ABOUT_XCARD"};
	    for (int i = 0; i < helpKeys.length; i++) {
		Component item = help.getMenuComponent(i);
		if (item != null) {
		    ((JMenuItem) item).setText(resources.getString(helpKeys[i]));
		}
	    }
	}
    }

    public static void setCardEngineInstance(CardEngine new_engine)
    {
	engine = new_engine;
	engine.addXCardEventListener(engineListener);
    }

    public static CardEngine getEngineInstance()
    {
	return engine;
    }

    public static PreferencesService getPreferencesService()
    {
	return ps;
    }

    public static AudioMaster getAudio()
    {
	if (audio == null) {
	    try {
		Resource resource = new Resource();
		audio = new AudioMaster();
		URL correctURL = 
		    resource.get("xcard/resources/corkpop2.wav");
		URL incorrectURL = 
		    resource.get("xcard/resources/guitarboing.wav");

		audio.setClip("CORRECT", correctURL);
		audio.setClip("INCORRECT", incorrectURL);
		audio.setVolume(ps.getVolume());
	    } catch(LineUnavailableException e) {
		Message.warningMessage("LINE_UNAVAILABLE");
	    } catch(Exception e) {
		Message.errorMessage(e);
	    }
	}
	return audio;
    }

    private void buildViewNames(ResourceBundle resources)
    {
	for (int i = 0; i < viewKeys.length; i++) {
	    views[i] = resources.getString(viewKeys[i]);
	}
    }

    public void about()
    {
	try {

	    Resource resource = new Resource();
	    String version = resource.getProperty("version");
	    String msg = new String("XCard Release " + version);
	    JOptionPane.showMessageDialog(null, msg, "Info",
					  JOptionPane.INFORMATION_MESSAGE);
	} catch (IOException e1) {
	    Message.errorMessage(e1);
	}
    }

    public void preferences()
    {
	String[] buttonRowObjects = {
	    "Ok"
	};
	JOptionPane.showOptionDialog(null,
				     preferences,
				     "Preferences",
				     JOptionPane.DEFAULT_OPTION,
				     JOptionPane.PLAIN_MESSAGE,
				     null,
				     buttonRowObjects,
				     null);

	preferences.fireUpdateEvent();
	try {
	    ps.commitChanges();
	} catch (IOException io) {
	    Message.errorMessage(io);
	}
    }

    public void quit()
    {
	try {
	    // Save any preferences
	    ps.commitChanges();

	    if (engine.isCardListModified()) {
		Message message = new Message();
		int saved = message.showUnSavedMessage(engine);
		if (saved == Message.YES || saved == Message.NO) {
		    exit();
		}
	    } else {
		exit();
	    }
	} catch(Exception e) {
	    Message.errorMessage(e);
	    exit();
	}
    }

    public static void main(String[] args)
    {
	try {
	    // Set up some Mac OS X settings 
	    if (Resource.isApple()) {
		System.setProperty("com.apple.mrj.application.apple.menu.about.name", "XCard");
		System.setProperty("apple.laf.useScreenMenuBar", "true");
		System.setProperty("apple.awt.showGrowBox", "true");
	    }

	    ps = new PreferencesService();
	    String value = ps.getLookAndFeel();
	    if (value != null && value.equals("Kunststoff")) {
		com.incors.plaf.kunststoff.KunststoffLookAndFeel kunststoffLnF
		    = new com.incors.plaf.kunststoff.KunststoffLookAndFeel();
		kunststoffLnF.setCurrentTheme(new com.incors.plaf.kunststoff.KunststoffTheme());
		UIManager.setLookAndFeel(kunststoffLnF);
		// XXXX Needed for WebStart
		UIManager.getLookAndFeelDefaults().put("ClassLoader", kunststoffLnF.getClass().getClassLoader());
	    }
	    XCard xc = new XCard();
	} catch (UnsupportedLookAndFeelException e) {
	    Message.errorMessage(e);
	} catch (Exception e) {
	    Message.errorMessage(e);
	}
    }
}


