/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header$
 */

package xcard;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.beans.*;
import java.util.*;

public class XCardEditPanel extends JPanel {
    JLabel editListLabel = null;

    JLabel listName = null;
    JList itemList = null;

    JLabel prefix = null;
    JTextField prefixText = null;

    JLabel word = null;
    JTextField wordText = null;

    JLabel transPrefix = null;
    JTextField transPrefixText = null;

    JLabel translation = null;
    JTextField translationText = null;
    
    JLabel comment = null;
    JTextField commentText = null;

    JLabel hint1 = null;
    JTextField hint1Text = null;

    JButton newList, delete, clear, add, update;

    private boolean fieldsComposed = false;

    public void composeFields()
    {
	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();

	JPanel panel = new JPanel();

	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	panel.setLayout(gbl);

	editListLabel = new JLabel(resources.getString("EDIT_WORDS"));
	Font font = new Font("Helvetica", Font.BOLD + Font.ITALIC, 16);
	editListLabel.setFont(font);

	listName = new JLabel();
	listName.setFont(font);

	JPanel p = new JPanel();
	p.add(editListLabel);
	p.add(listName);
	

	JSeparator sep = new JSeparator();
	
	// row 1
	gbc.anchor = GridBagConstraints.WEST;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(p, gbc);

	gbc.anchor = GridBagConstraints.NORTH;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 0, 10, 0);
	panel.add(sep, gbc);

	// row 2
	itemList = new JList(new DefaultListModel());
	EntryListCellRenderer renderer = new EntryListCellRenderer();
	itemList.setCellRenderer(renderer);
	
	ListSelectionModel selectModel = itemList.getSelectionModel();
	selectModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	itemList.addListSelectionListener(new ListSelectionListener() {
		public void valueChanged(ListSelectionEvent e)
		{
		    Entry entry = (Entry) itemList.getSelectedValue();
		    setInput(entry);
		}
	    });

	itemList.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    if(e.getClickCount() > 0) {
			Entry entry = (Entry) itemList.getSelectedValue();
			setInput(entry);
		    }
		}
	    });


	itemList.setFixedCellWidth(320);	
	itemList.setVisibleRowCount(6);

	JPanel listPanel = new JPanel();
	JScrollPane sp = new JScrollPane(itemList);
	listPanel.add(sp);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(listPanel, gbc);
	
	// row 3
	gbc.gridwidth = 1;
	prefix  = new JLabel(resources.getString("PREFIX:"));
	prefixText = new JTextField(4);
	addAccentAction(prefixText);

	word = new JLabel(resources.getString("WORD:"));
	wordText = new JTextField(15);
	addAccentAction(wordText);

	panel.add(prefix, gbc);
	panel.add(prefixText, gbc);
	panel.add(Box.createHorizontalStrut(10));
	
	panel.add(word, gbc);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(wordText, gbc);

	// row 4
	gbc.gridwidth = 1;
	transPrefix = new JLabel(resources.getString("TRANSLATION_PREFIX:"));
	transPrefixText = new JTextField(4);
	addAccentAction(transPrefixText);

	translation = new JLabel(resources.getString("TRANSLATION:"));
	translationText = new JTextField(15);
	translationText.addActionListener(new AddActionListener());
	addAccentAction(translationText);
	
	panel.add(transPrefix, gbc);
	panel.add(transPrefixText, gbc);
	panel.add(Box.createHorizontalStrut(10));

	panel.add(translation, gbc);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(translationText, gbc);

	// row 5
	gbc.gridwidth = 1;
	comment = new JLabel(resources.getString("COMMENT:"));
	commentText = new JTextField(10);
	commentText.addActionListener(new AddActionListener());
	addAccentAction(commentText);
	
	panel.add(comment, gbc);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(commentText, gbc);

	// row 6
	newList = new JButton(resources.getString("NEW_LIST"));
	newList.addActionListener(new CreateNewListListener());
	delete = new JButton(resources.getString("DELETE"));
	delete.addActionListener(new DeleteActionListener());

	clear = new JButton(resources.getString("CLEAR"));
	clear.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    setInput(null);
		}
	    });

	add = new JButton(resources.getString("ADD"));
	add.addActionListener(new AddActionListener());
	
	update = new JButton(resources.getString("UPDATE"));
	update.addActionListener(new UpdateActionListener());

	JPanel buttonPanel = new JPanel();
	buttonPanel.add(newList);
	buttonPanel.add(delete);
	buttonPanel.add(update);
	buttonPanel.add(clear);
	buttonPanel.add(add);

	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(buttonPanel, gbc);

	add(panel);
	fieldsComposed = true;
	updateFieldValues();
    }

    protected void addAccentAction(JTextField textField)
    {
	KeyStroke keyUp = 
	    KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0);
	KeyStroke keyDown = 
	    KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0);

	AccentActionListener accentAction = new AccentActionListener();
	accentAction.setJTextField(textField);
	textField.registerKeyboardAction(accentAction, "next", keyUp,
					 JComponent.WHEN_FOCUSED);

	textField.registerKeyboardAction(accentAction, "prev", keyDown,
					 JComponent.WHEN_FOCUSED);

    }

    public boolean areFieldsComposed()
    {
	return fieldsComposed;
    }

    private class AddActionListener implements ActionListener {

	public void actionPerformed(ActionEvent e)
	{
	    CardEngine engine = XCard.getEngineInstance();
	    engine.setCardListModified(true);
	    
	    Entry entry = gatherInput();
	    DefaultListModel model = (DefaultListModel) itemList.getModel();
	    model.addElement(entry);
	    itemList.setSelectedValue(entry, true);
	    engine.add(entry);
	    CardList clist = engine.getCardList();
	    if (clist.getName() == null) {
		Message msg = new Message();
		String listName = msg.showMissingListNameMessage(engine);
		if (listName != null) {
		    clist.setName(listName);
		    updateFieldValues();
		}
	    }

	    firePropertyChange("ENGINE_ADD_ENTRY", null, engine);
	    setInput(null);
	    prefixText.grabFocus();
	}
    }
			       
    private class UpdateActionListener implements ActionListener {
	
	public void actionPerformed(ActionEvent e)
	{
	    CardEngine engine = XCard.getEngineInstance();
	    engine.setCardListModified(true);
	    Entry entry = gatherInput();
	    int index = itemList.getSelectedIndex();

	    DefaultListModel model = (DefaultListModel) itemList.getModel();
	    Entry modelEntry = (Entry) model.getElementAt(index);

	    CardList list = engine.getCardList();
	    Entry engineEntry = list.get(modelEntry.getFullWord());
	    engineEntry.set(entry);

	    model.setElementAt(entry, index);
	    firePropertyChange("ENGINE_UPDATE_ENTRY", null, engine);
	    setInput(null);
	    prefixText.grabFocus();
	}
    }

    private class DeleteActionListener implements ActionListener {

	public void actionPerformed(ActionEvent e)
	{
	    CardEngine engine = XCard.getEngineInstance();
	    engine.setCardListModified(true);
	    int index = itemList.getSelectedIndex();
	    DefaultListModel model = (DefaultListModel) itemList.getModel();
	    Entry entry = (Entry) model.elementAt(index);
	    CardList clist = engine.getCardList();
	    clist.remove(entry);
	    model.remove(index);

	    firePropertyChange("ENGINE_REMOVE_ENTRY", null, engine);
	    if (index < clist.size()) {
		itemList.setSelectedIndex(index);
	    }
	    setInput(null);
	}
    }
	    
    private class CreateNewListListener implements ActionListener {

	public void actionPerformed(ActionEvent e)
	{
	    boolean createList = false;
	    CardEngine engine = XCard.getEngineInstance();
	    if (engine.isCardListModified()) {
		Message message = new Message();
		int saved = message.showUnSavedMessage(engine);
		if (saved == Message.YES || saved == Message.NO) {
		    createList = true;
		}
	    } else {
		createList = true;
	    }
	    
	    if (createList) {
		engine = new CardEngine();
		engine.setCardListModified(true);

		Message msg = new Message();
		String listName = msg.showMissingListNameMessage(engine);
		
		if (listName != null) {
		    CardList clist = engine.getCardList();
		    clist.setName(listName);
		    XCard.setCardEngineInstance(engine);
		    updateFieldValues();
		    firePropertyChange("ENGINE_NEW", null, engine);
		}
		prefixText.grabFocus();
	    }
	}

    }

    private Entry gatherInput()
    {
	Entry entry = null;
	String prefix = prefixText.getText();
	String word = wordText.getText();
		    
	String transPrefix = transPrefixText.getText();
	String translation = translationText.getText();

	String comment = commentText.getText();

	if (word.length() > 0 && translation.length() > 0) {
	    entry = new Entry();
	}

	if (entry != null) {
	    entry.setPrefix(prefix);
	    entry.setWord(word);
	    entry.setTranslationPrefix(transPrefix);
	    entry.setTranslation(translation);
	    entry.setComment(comment);
	}

	return entry;
    }

    public void updateFieldValues()
    {
	if (fieldsComposed) {
	    CardEngine engine = XCard.getEngineInstance();
	    CardList clist = engine.getCardList();
	    Entry[] entries = clist.toSortedArrayByName();
	    Entry selected = engine.getCurrentEntry();
	    ListSelectionModel selectModel = itemList.getSelectionModel();
	    DefaultListModel model = new DefaultListModel();
	    for (int i = 0; i < entries.length; i++) {
		if (selected != null && selected.equals(entries[i])) {
		    selectModel.setAnchorSelectionIndex(i);
		}
		model.addElement(entries[i]);
	    }
	    itemList.setModel(model);
	    listName.setText(clist.getName());
	}
    }

    public void setInput(Entry e)
    {
	String prefix = "";
	String word = "";
	String transPrefix = "";
	String translation = "";
	String comment = "";

	if (e != null) {
	    prefix = e.getPrefix();
	    word = e.getWord();
	    transPrefix = e.getTranslationPrefix();
	    translation = e.getTranslation();
	    comment = e.getComment();
	}
	prefixText.setText(prefix);
	wordText.setText(word);

	transPrefixText.setText(transPrefix);
	translationText.setText(translation);
	
	commentText.setText(comment);
    }
    
    public void updateLocalizedText()
    {
	if (fieldsComposed) {
	    PreferencesService ps = XCard.getPreferencesService();
	    ResourceBundle resources = ps.getResourceBundle();
	
	    editListLabel.setText(resources.getString("EDIT_WORDS"));
	    transPrefix.setText(resources.getString("TRANSLATION_PREFIX:"));
	    translation.setText(resources.getString("TRANSLATION:"));
	    word.setText(resources.getString("WORD:"));
	    comment.setText(resources.getString("COMMENT:"));
	    newList.setText(resources.getString("NEW_LIST"));
	    delete.setText(resources.getString("DELETE"));
	    clear.setText(resources.getString("CLEAR"));
	    add.setText(resources.getString("ADD"));
	    update.setText(resources.getString("UPDATE"));
	}
    }
}
