/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000, 2001
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/XCardLookupPanel.java,v 1.10 2003/12/08 00:06:15 ellis Exp ellis $
 */

package xcard;

import java.awt.*;
import java.awt.event.*;
import java.io.IOException;
import java.io.File;
import java.util.*;
import javax.swing.*;
import javax.sound.sampled.*;

public class XCardLookupPanel extends JPanel {
    JLabel listName;
    JLabel title = null;
    JLabel from_label = null;
    JLabel translateToLabel = null;
    JComboBox transDirection = null;
    JTextField from = null;
    JButton translate = null;
    JButton addToList = null;
    JButton create = null;
    JButton speak = null;

    JLabel word_label = null;
    JTextField prefix = null;
    JTextField word = null;


    JLabel status = null;
    Lookup lookup = null;
    Parlez parlez = null;
    private SwingWorker worker = null;

    private boolean composed = false;

    protected void addAccentAction(JTextField textField)
    {
	KeyStroke keyUp = 
	    KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0);
	KeyStroke keyDown = 
	    KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0);

	AccentActionListener accentAction = new AccentActionListener();
	accentAction.setJTextField(textField);
	textField.registerKeyboardAction(accentAction, "next", keyUp,
					 JComponent.WHEN_FOCUSED);

	textField.registerKeyboardAction(accentAction, "prev", keyDown,
					 JComponent.WHEN_FOCUSED);

    }

    public boolean areFieldsComposed()
    {
	return composed;
    }

    public void composeFields()
    {
	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	gbc.anchor = GridBagConstraints.NORTHWEST;

	JPanel panel = new JPanel();
	panel.setLayout(gbl);

	composeTitle(panel, gbc);
	composeLookup(panel, gbc);
	add(panel);

	lookup = new Lookup();
	parlez = new Parlez();

	lookup.addXCardEventListener(new XCardEventListener()
	    {
		public void actionEvent(XCardEvent event)
		{
		    String name = event.getName();
		    if (name.equals("LOOKUP_STATUS")) {
			Lookup l = (Lookup) event.getSource();
			status.setText(l.getStatus());
		    } 
		}

	    });

	parlez.addXCardEventListener(new XCardEventListener()
	    {
		public void actionEvent(XCardEvent event)
		{
		    String name = event.getName();
		    if (name.equals("LOOKUP_STATUS")) {
			Parlez p = (Parlez) event.getSource();
			status.setText(p.getStatus());
		    } 
		}

	    });
	
	composed = true;
    }

    public void composeTitle(JPanel panel, GridBagConstraints gbc)
    {
	Font font = new Font("Helvetica", Font.BOLD + Font.ITALIC, 16);
	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	title = new JLabel(resources.getString("LOOKUP"));
	title.setFont(font);

	CardEngine engine = XCard.getEngineInstance();
	if (engine != null) {
	    CardList clist = engine.getCardList();
	    listName = new JLabel(clist.getName());
	} else {
	    listName = new JLabel();
	}
	listName.setFont(font);

	JSeparator sep = new JSeparator();
	sep.setPreferredSize(new Dimension(450, 5));
	
	// row 1
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	JPanel p = new JPanel();
	p.add(title);
	p.add(listName);
	panel.add(p, gbc);

	gbc.anchor = GridBagConstraints.NORTH;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 0, 10, 0);
	panel.add(sep, gbc);

    }

    private class LookupWorker extends SwingWorker
    {
	public Object construct()
	{
	    String value = null;
	    try {
		PreferencesService ps = XCard.getPreferencesService();
		String translateTo = ps.getTranslationDirection();
		String inputLang;


		if (translateTo.equals("fr")) {
		    inputLang = "en";
		} else {
		    inputLang = "fr";
		}

		lookup.setInputLanguage(inputLang);
		value = lookup.getTranslation(from.getText());
		if (worker == this) {
		    setTranslatedText(value);
		}

	    } catch (java.net.UnknownHostException e) {
		status.setText("Unable to connect to world.altavista.com");
	    } catch (IOException e) {
		status.setText(e.getMessage());
	    }
	    return value;
	}
    }

    private class ParlezWorker extends SwingWorker
    {
	public Object construct()
	{
	    String value = null;
	    File file = null;
	    try {
		String p = prefix.getText();
		String w = word.getText();
		String data = p + " " + w;
		AudioMaster audio = XCard.getAudio();
		boolean audio_enabled = audio.isEnabled();

		if (audio_enabled) {
		    if (! audio.isClipCached(data)) {
			PreferencesService ps = XCard.getPreferencesService();
			String translateTo = ps.getTranslationDirection();

			file = parlez.getWavFile(data, translateTo);
			audio.setClip(data, file.toURL());
		    }
		    audio.playClip(data);
		} else {
		    status.setText("Sound has been disabled in Preferences");
		}
		
	    } catch (java.net.UnknownHostException e) {
		status.setText("Unable to connect to world.altavista.com");
	    } catch (IOException e) {
		status.setText(e.getMessage());
	    } catch(UnsupportedAudioFileException e) {
		status.setText(e.getMessage());
	    } catch (LineUnavailableException e) {
		status.setText("Unable to play sound");
	    } finally {
		if (file != null) {
		    if (! file.delete()) {
			file.deleteOnExit();
		    }
		}
	    }
	    return value;
	}
    }

    private String getPrefix(String text)
    {
	String value = null;
	if (text != null) {
	    String[] prefixes = {"la", "le", "les", "un", "une", "il", "ils",
				 "elle", "elles", "tu", "vous", "nous"
	    };
	    boolean prefix = false;
	    int i = 0;

	    while (i < prefixes.length && ! prefix) {
		String v = text.toLowerCase();
		prefix = v.startsWith(prefixes[i] + " ");
		if (! prefix) {
		    i++;
		}
	    }
	    if (prefix) {
		value = text.substring(0, prefixes[i].length());
	    }
	}
	return value;
    }

    private synchronized void setTranslatedText(String text)
    {
	String currentValue = word.getText();
	if (currentValue != null) {
	    word.setText("");
	    prefix.setText("");
	}
	String prefix_s = getPrefix(text);
	if (prefix_s != null) {
	    prefix.setText(prefix_s);
	    text  = text.substring(prefix_s.length());
	    text = text.trim();
	}
	word.setText(text);
    }

    private void performLookup()
    {
	worker = new LookupWorker();
	worker.start();
    }

    private void performParlez()
    {
	worker = new ParlezWorker();
	worker.start();
    }

    private class ExecuteTranslationListener implements ActionListener 
    {
	public void actionPerformed(ActionEvent e) {
	    String value = from.getText();
	    if (value != null && value.length() > 0) {
		performLookup();
	    }
	}
    }

    private class ParlezTranslationListener implements ActionListener 
    {
	public void actionPerformed(ActionEvent e) {
	    String value = word.getText();
	    if (value != null && value.length() > 0) {
		performParlez();
	    }
	}
    }


    private class AddToListListener implements ActionListener
    {
	public void actionPerformed(ActionEvent e) {
	    boolean hasName = false;
	    CardEngine engine = XCard.getEngineInstance();

	    CardList clist = engine.getCardList();
	    if (clist.getName() == null) {
		Message msg = new Message();
		String listName = msg.showMissingListNameMessage(engine);
		if (listName != null) {
		    clist.setName(listName);
		    updateValues();
		    hasName = true;
		}
	    } else {
		hasName = true;
	    }

	    if (hasName) {
		String prefix_v = prefix.getText();
		prefix_v = prefix_v.trim();

		String word_v = word.getText();
		word_v = word_v.trim();

		String from_v = from.getText();
		from_v = from_v.trim();

		if (word_v != null && word_v.length() > 0) {
		    Entry entry = new Entry();
		    if (prefix_v != null && prefix_v.length() > 0) {
			entry.setPrefix(prefix_v);
			int i = from_v.indexOf(" ");
			if (i >= 0) {
			    String transPrefix = from_v.substring(0, i);
			    entry.setTranslationPrefix(transPrefix);
			    String translation = from_v.substring(i + 1, 
								  from_v.length());
			    entry.setTranslation(translation);
			}
		    } else {
			entry.setTranslation(from_v);
		    }
		    entry.setWord(word_v);

		    // XXXX Re-versing if we are using english.  I only
		    // want to keep one file format
		    PreferencesService ps = XCard.getPreferencesService();
		    String translateTo = ps.getTranslationDirection();
		    if (translateTo.equals("en")) {
			String t = entry.getTranslation();
			String w = entry.getWord();
			
			entry.setTranslation(w);
			entry.setWord(t);
		    }

		    engine.setCardListModified(true);
		    engine.add(entry);
		    firePropertyChange("LOOKUP_ADD_ENTRY", null, this);
		    
		    clearFields();
		    status.setText("Added " + entry.getFullWord());
		    from.grabFocus();
		}
	    }
	}
    }

    private void clearFields()
    {
	from.setText("");
	prefix.setText("");
	word.setText("");
    }

    private class CreateNewListListener implements ActionListener {

	public void actionPerformed(ActionEvent e)
	{
	    boolean createList = false;
	    CardEngine engine = XCard.getEngineInstance();
	    if (engine.isCardListModified()) {
		Message message = new Message();
		int saved = message.showUnSavedMessage(engine);
		if (saved == Message.YES || saved == Message.NO) {
		    createList = true;
		}
	    } else {
		createList = true;
	    }
	    
	    if (createList) {
		engine = new CardEngine();
		engine.setCardListModified(true);

		Message msg = new Message();
		String listName = msg.showMissingListNameMessage(engine);
		
		if (listName != null) {
		    CardList clist = engine.getCardList();
		    clist.setName(listName);
		    XCard.setCardEngineInstance(engine);
		    updateValues();
		    status.setText("Created " + listName);
		    firePropertyChange("LOOKUP_ENGINE_NEW", null, engine);
		}
	    }
	}

    }

    private String[] getTransChoices(String translateTo, 
				     ResourceBundle resources)
    {
	String[] choices = new String[2];
	if (translateTo.equals("fr")) {
	    choices[0] = resources.getString("FRENCH");
	    choices[1] = resources.getString("ENGLISH");
	} else {
	    choices[0] = resources.getString("ENGLISH");
	    choices[1] = resources.getString("FRENCH");
	}
	return choices;
    }

    private JComboBox composeTransDirection(PreferencesService ps)

    {
	String translateTo = ps.getTranslationDirection();
	final ResourceBundle resources = ps.getResourceBundle();

	transDirection = new JComboBox(getTransChoices(translateTo, resources));
	transDirection.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    String transTo = (String) transDirection.getSelectedItem();
		    if (transTo != null) {
			if (transTo.equals(resources.getString("FRENCH"))) {
			    transTo = "fr";
			} else {
			    transTo = "en";
			}
			PreferencesService ps = XCard.getPreferencesService();
			ps.setTranslationDirection(transTo);
			firePropertyChange("PREFERENCES_TRANSDIR_CHANGE", null, this);
			updateFromLabel(transTo, resources);
		    }
		}
	    });

	return transDirection;
    }

    public void composeLookup(JPanel panel, GridBagConstraints gbc)
    {
	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	
	ExecuteTranslationListener eql = new ExecuteTranslationListener();
	ParlezTranslationListener ptl = new ParlezTranslationListener();
	AddToListListener atl = new AddToListListener();
	CreateNewListListener cnl = new CreateNewListListener();

	transDirection = composeTransDirection(ps);

	translate = new JButton(resources.getString("TRANSLATE"));
	translate.setDefaultCapable(true);
	translate.addActionListener(eql);
	KeyStroke keyEnter = KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0);
	translate.registerKeyboardAction(eql, "translate", keyEnter,
					 JComponent.WHEN_FOCUSED);
	
	
	addToList = new JButton(resources.getString("ADD_TO_LIST"));
	addToList.addActionListener(atl);
	addToList.registerKeyboardAction(atl, "add", keyEnter,
					 JComponent.WHEN_FOCUSED);

	create = new JButton(resources.getString("CREATE_NEW_LIST"));
	create.addActionListener(cnl);
	create.registerKeyboardAction(cnl, "create", keyEnter,
				      JComponent.WHEN_FOCUSED);
	
	status = new JLabel("", SwingConstants.CENTER);

	String translateTo = ps.getTranslationDirection();
	String wordLabel = "FR_WORD:";
	if (translateTo.equals("fr")) {
	     wordLabel = "EN_WORD:";
	} 
	from_label = new JLabel(resources.getString(wordLabel));
	
	from = new JTextField(19);
	addAccentAction(from);
	from.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    performLookup();
		}
	    });
			       
	word_label = new JLabel(resources.getString("TRANSLATION:"));
	prefix = new JTextField(4);
	addAccentAction(prefix);

	word = new JTextField(15);
	addAccentAction(word);
	word.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    String value = word.getText();
		    if (value != null && value.length() > 0) {
			performParlez();
		    }
		}
	    });
	
	speak = new JButton(resources.getString("SPEAK"));
	speak.setDefaultCapable(true);
	speak.addActionListener(ptl);
	speak.registerKeyboardAction(ptl, "speak", keyEnter,
				     JComponent.WHEN_FOCUSED);

	gbc.gridwidth = 1;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	translateToLabel = new JLabel(resources.getString("TRANSLATE_TO:"));
	panel.add(translateToLabel, gbc);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(transDirection, gbc);

	gbc.gridwidth = 1;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	panel.add(from_label, gbc);

	gbc.gridwidth = 2;
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.NONE;
	panel.add(from, gbc);

	gbc.anchor = GridBagConstraints.WEST;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(translate, gbc);

	gbc.fill = GridBagConstraints.NONE;
	gbc.gridwidth = 1;

	gbc.anchor = GridBagConstraints.WEST;
	panel.add(word_label, gbc);
	panel.add(prefix, gbc);
	panel.add(word, gbc);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(speak, gbc);

	gbc.anchor = GridBagConstraints.CENTER;
	gbc.fill = GridBagConstraints.NONE;
	//panel.add(create, gbc);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(addToList, gbc);



	gbc.anchor = GridBagConstraints.CENTER;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbc.fill = GridBagConstraints.HORIZONTAL;
       	panel.add(status, gbc);
    }

    public void updateValues()
    {
	if (composed) {
	    CardEngine engine = XCard.getEngineInstance();
	    if (engine != null) {
		CardList clist = engine.getCardList();
		listName.setText(clist.getName());
	    }
	}
    }

    public void updateLocalizedText()
    {
	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	String translateTo = ps.getTranslationDirection();

	if (title != null) {
	    title.setText(resources.getString("LOOKUP"));
	}
	if (translate != null) {
	    translate.setText(resources.getString("TRANSLATE"));
	}
	if (addToList != null) {
	    addToList.setText(resources.getString("ADD_TO_LIST"));
	}
	if (create != null) {
	    create.setText(resources.getString("CREATE_NEW_LIST"));
	}

	updateFromLabel(translateTo, resources);

	if (word_label != null) {
	    word_label.setText(resources.getString("TRANSLATION:"));
	}
	if (speak != null) {
	    speak.setText(resources.getString("SPEAK"));
	}

	if (transDirection != null) {
	    transDirection.removeAllItems();
	    String[] choices = getTransChoices(translateTo, resources);
	    for (int i = choices.length -1; i >= 0; i--) {
		transDirection.insertItemAt(choices[i], 0);
	    }
	    transDirection.setSelectedIndex(0);
	}
	if (translateToLabel != null) {
	    translateToLabel.setText(resources.getString("TRANSLATE_TO:"));
	}
    }

    private void updateFromLabel(String translateTo, ResourceBundle resources)
    {
	if (from_label != null) {
	    String wordLabel = "FR_WORD:";
	    if (translateTo.equals("fr")) {
		wordLabel = "EN_WORD:";
	    } 
	    from_label.setText(resources.getString(wordLabel));
	}
    }

}
