/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000, 2001
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/XCardPanel.java,v 1.37 2003/12/08 00:06:15 ellis Exp ellis $
 */

package xcard;

import java.io.IOException;
import java.net.URL;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;
import javax.sound.sampled.LineUnavailableException;
import org.w3c.dom.*;


public class XCardPanel extends JPanel {

    JLabel listName = null;
    JLabel word = null;
    JTextField translationText = null;
    JLabel scoreLabel = null;
    JLabel comment = null;
    JCheckBox excellent = null, good = null, average = null,
	poor = null;
    JLabel picture = new JLabel();
    JProgressBar progress;
    JLabel total;

    private int progressValue = 0;
    private int pictureCounter = 0;

    JCheckBox repeated = null;

    JLabel challengeLabel, translation, title, commentLabel;
    JButton next, checkAnswer;

    int inCorrectCounter = 0;

    int nextEntryType = CardEngine.REPEATED;

    Resource resource = new Resource();
 
    private Color defaultColor = new Color(102,102,153);

    private String state = "correct";

    private ArrayList pictures = new ArrayList();

    private boolean fieldsComposed = false;
    
    public void composeFields()
    {
	AudioMaster audio =  XCard.getAudio();
	PreferencesService ps = XCard.getPreferencesService();
	if (ps.isSoundEnabled() && resource.JVMsupportsSound()) {
	    audio.enableSound();
	} else {
	    audio.disableSound();
	}
	
	JPanel panel = new JPanel();

	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	gbc.anchor = GridBagConstraints.NORTHWEST;

	panel.setLayout(gbl);
	
	buildPictureList();
	composeChallenge(panel, gbc);
	fieldsComposed = true;
	updateValues();
    }

    public boolean areFieldsComposed()
    {
	return fieldsComposed;
    }

    private void buildFaceIcon(String pic, JCheckBox cbox, 
			       CheckBoxListener cboxListener)
    {
	String prefix = "xcard/resources/";
	URL icon = resource.get(prefix + pic + "-small.png");
	cbox.setIcon(new ImageIcon(icon));
	icon = resource.get(prefix + pic + "-roll.png");
	//cbox.setRolloverIcon(new ImageIcon(icon));
	icon = resource.get(prefix + pic + "-unselected.png");
	cbox.setIcon(new ImageIcon(icon));
	icon = resource.get(prefix + pic + "-selected.png");
	cbox.setSelectedIcon(new ImageIcon(icon));
	cbox.addItemListener(cboxListener);
    }

    private class CheckBoxListener implements ItemListener {

	public void itemStateChanged(ItemEvent event) {
	    JCheckBox b = (JCheckBox) event.getItem();
	    CardEngine engine = XCard.getEngineInstance();
	    CardList list = engine.getCardList();

	    int type = -1;
	    if (b == excellent) {
		type = State.EXCELLENT;
	    } else if (b == good) {
		type = State.GOOD;
	    } else if (b == average) {
		type = State.AVERAGE;
	    } else if (b == poor) {
		type = State.POOR;
	    }

	    if (b.isSelected()) {
		list.enable(type);
	    } else {
		list.disable(type);
	    }

	    // If we have a null entry bop the next word on to the screen
	    Entry e = engine.getCurrentEntry();
	    if (e == null && b.isSelected()) {
		updateValues();
	    } 
	}
    }

    public void composeTotals(JPanel panel, GridBagConstraints gbc)
    {
	URL icon = null;

	JPanel labels = new JPanel(new FlowLayout(FlowLayout.CENTER));
	EmptyBorder border = new EmptyBorder(5,5,5,5);
	labels.setBorder(border);
	Font font = new Font("Helvetica", Font.PLAIN, 14);

	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	total = new JLabel(resources.getString("TOTALS:"));
	total.setFont(font);

	CheckBoxListener cboxListener = new CheckBoxListener();

	excellent = new JCheckBox(getEntryStateCount(State.EXCELLENT),
				  true);
	excellent.setToolTipText("Excellent");
	excellent.setFont(font);
	buildFaceIcon("face-win", excellent, cboxListener);

	good = new JCheckBox(getEntryStateCount(State.GOOD), true);
	good.setToolTipText("Good");
	good.setFont(font);
	buildFaceIcon("face-smile", good, cboxListener);
	
	average = new JCheckBox(getEntryStateCount(State.AVERAGE), true);
	average.setToolTipText("Average");
	average.setFont(font);
	buildFaceIcon("face-worried", average, cboxListener);

	poor = new JCheckBox(getEntryStateCount(State.POOR), true);
	poor.setToolTipText("Poor");
	poor.setFont(font);
	buildFaceIcon("face-sad", poor, cboxListener);
	
	gbc.gridwidth = 1;
	//panel.add(total, gbc);
	//panel.add(Box.createHorizontalStrut(10));

	labels.add(total);
	labels.add(excellent);
	labels.add(Box.createHorizontalStrut(10));
	labels.add(good);
	labels.add(Box.createHorizontalStrut(10));
	labels.add(average);
	labels.add(Box.createHorizontalStrut(10));
	labels.add(poor);
	
	gbc.anchor = GridBagConstraints.CENTER;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(labels, gbc);
    }

    public void composeProgressBar(JPanel panel, GridBagConstraints gbc)
    {
	gbc.anchor = GridBagConstraints.CENTER;
	gbc.gridwidth = GridBagConstraints.REMAINDER;

	CardEngine engine = XCard.getEngineInstance();
	progress = new JProgressBar(0, engine.getCardList().size());
	progress.setPreferredSize(new Dimension(5, 5));
	panel.add(progress, gbc);
    }

    private String getEntryStateCount(int entry_state)
    {
	String result;
	CardEngine engine = XCard.getEngineInstance();
	CardList clist = engine.getCardList();
	if (engine != null) {
	    Integer i = 
		new Integer(clist.size(entry_state));
	    result = i.toString();
	} else {
	    result = "0";
	}
	return result;
    }

    public void composeChallenge(JPanel panel, GridBagConstraints gbc)
    {

	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	challengeLabel = new JLabel(resources.getString("WORD:"));
	word = new JLabel();
	word.setPreferredSize(new Dimension(230, 15));

	translation = new JLabel(resources.getString("TRANSLATION:"));

	translationText = new JTextField(15);

	translationText.setPreferredSize(new Dimension(20, 20));
	translationText.addActionListener(new CompareHandler());

	KeyStroke keyUp = 
	    KeyStroke.getKeyStroke(KeyEvent.VK_UP, 0);
	KeyStroke keyDown = 
	    KeyStroke.getKeyStroke(KeyEvent.VK_DOWN, 0);

	AccentActionListener accent = new AccentActionListener();
	accent.setJTextField(translationText);

	translationText.registerKeyboardAction(accent,
					       "next", keyUp,
					       JComponent.WHEN_FOCUSED);

	translationText.registerKeyboardAction(accent,
					       "prev", keyDown,
					       JComponent.WHEN_FOCUSED);

	scoreLabel = new JLabel();

	JSeparator sep = new JSeparator();
	sep.setPreferredSize(new Dimension(450, 5));


	title = new JLabel(resources.getString("TRANSLATE"));

	CardEngine engine = XCard.getEngineInstance();
	if (engine != null) {
	    CardList clist = engine.getCardList();
	    listName = new JLabel(clist.getName());
	} else {
	    listName = new JLabel();
	}

	commentLabel = new JLabel(resources.getString("COMMENT:"));
	comment = new JLabel();
	
	Font font = new Font("Helvetica", Font.BOLD + Font.ITALIC, 16);
	title.setFont(font);
	listName.setFont(font);

	// row 1
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	JPanel p = new JPanel();
	p.add(title);
	p.add(listName);
	panel.add(p, gbc);

	gbc.anchor = GridBagConstraints.NORTH;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 0, 10, 0);
	panel.add(sep, gbc);

	composeTotals(panel, gbc);
	composeProgressBar(panel, gbc);
	
	// row 2
	gbc.gridwidth = 1;
	challengeLabel.setHorizontalTextPosition(SwingConstants.CENTER);
	panel.add(challengeLabel, gbc);
	
	panel.add(Box.createHorizontalStrut(10));
	panel.add(word, gbc);

	panel.add(Box.createHorizontalStrut(10));
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	scoreLabel.setHorizontalAlignment(SwingConstants.CENTER);
	panel.add(scoreLabel, gbc);


	// row 3
	gbc.gridwidth = 1;
	panel.add(commentLabel, gbc);
	panel.add(Box.createHorizontalStrut(10));
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(comment, gbc);

	gbc.gridwidth = 1;
	panel.add(translation, gbc);
	panel.add(Box.createHorizontalStrut(10));
	panel.add(translationText, gbc);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(Box.createHorizontalStrut(10), gbc);

	// row 4
	next = new JButton(resources.getString("SKIP"));
	next.addActionListener(new SkipHandler());

	checkAnswer = new JButton(resources.getString("CHECK_ANSWER"));

	checkAnswer.addActionListener(new CompareHandler());

	gbc.gridwidth = 1;
	panel.add(next, gbc);
	panel.add(Box.createHorizontalStrut(10));
	panel.add(checkAnswer, gbc);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(Box.createHorizontalStrut(10), gbc);
	
	// row 5
	repeated = new JCheckBox(resources.getString("REPEAT_LESSER"));
	repeated.setSelected(true);
	repeated.addItemListener(new ItemListener() {
		public void itemStateChanged(ItemEvent event)
		{
		    if (repeated.isSelected()) {
			nextEntryType = CardEngine.REPEATED;
		    } else {
			nextEntryType = CardEngine.NON_REPEATED;
		    }
		}
	    });

	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(repeated, gbc);

	// row 6
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	picture.setPreferredSize(new Dimension(300, 190));
	// Set the picture
	setNextPicture();
	picture.setHorizontalAlignment(SwingConstants.CENTER);
	panel.add(picture, gbc);

	add(panel);
    }
 
    private void setNextPicture()
    {
	Picture pic = pick_a_picture();
	URL image = resource.get("pics/" + pic.getSrc());
	if (image == null) {
	    Message.errorMessage("Null " + pic.getSrc() + 
				 "title: " + pic.getTitle());
	}
	picture.setIcon(new ImageIcon(image));
    }

    private void setScoreImage(Entry entry)
    {
	int score = -1;
	int state = State.POOR;

	if (entry != null) {
	    score = entry.getScore();
	    state = entry.getState();
	}
	String scoreIcon = "face-sad.png";
	String tooltip = "Unknown word";
	
	switch (state) {
	    
	case State.POOR:
	    scoreIcon = "face-sad.png";
	    tooltip = "Unknown word";
	    break;

	case State.AVERAGE:
	    scoreIcon = "face-worried.png";
	    tooltip = "Word not really known";
	    break;

	case State.GOOD:
	    scoreIcon = "face-smile.png";
	    tooltip = "Familiar with the word";
	    break;

	case State.EXCELLENT:
	    scoreIcon = "face-win.png";
	    tooltip = "C'est magnifique!";
	    break;
	}

	URL icon = resource.get("xcard/resources/" + scoreIcon);
	scoreLabel.setIcon((new ImageIcon(icon)));
	scoreLabel.setToolTipText(tooltip);

	Dimension size = getSize();
	paintImmediately(0, 0, size.width, size.height);
    }
    
    private void setState(String state)
    {
	this.state = state;
    }

    private boolean isIncorrectState()
    {
	boolean return_value = false;
	if (state.equals("incorrect")) {
	    return_value = true;
	}
	return return_value;
    }

    private boolean isCorrectState()
    {
	boolean return_value = isIncorrectState();
	return ! return_value;
    }

    private void incorrectResponse(Entry entry)
    {
	int oldState = entry.getState();

	entry.incrementIncorrect();
	int newState = entry.getState();
		    
	if (oldState != newState) {
	    setScoreImage(entry);			
	}

	PreferencesService ps = XCard.getPreferencesService();
	String transdir = ps.getTranslationDirection();
	if (transdir.equals("fr")) {
	    word.setText(entry.getFullWord());
	} else {
	    word.setText(entry.getFullTranslation());
	}
	word.setForeground(Color.red);
	translationText.setText("");

	AudioMaster audio =  XCard.getAudio();
	audio.playClip("INCORRECT");

	CardEngine engine = XCard.getEngineInstance();
	engine.repositionEntry();
	setState("incorrect");

	inCorrectCounter++;
    }

    private void correctResponse(Entry entry)
    {
	int oldState = entry.getState();

	if (isCorrectState()) {
	    entry.incrementCorrect();
	    
	    pictureCounter++;
	    if ((pictureCounter % 4) == 0) {
		setNextPicture();
	    }

	} else {
	    setState("correct");
	}
	
	int newState = entry.getState();

	boolean transistion = false;
	if (oldState != newState) {
	    transistion = true;
	}
	
	if (transistion) {
	    setScoreImage(entry);
	    try {
		Thread.currentThread().sleep(600);
	    } catch (InterruptedException e) {
		Message.errorMessage(e);
	    }
	}
	CardEngine engine = XCard.getEngineInstance();
	engine.repositionEntry();
    }

    private class SkipHandler implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    if (isIncorrectState()) {
		setState("correct");
	    }
	    nextEntry();
	}
    }

    private class CompareHandler implements ActionListener  {
	public void actionPerformed(ActionEvent e)
	{
	    String translation = translationText.getText().trim();
	    CardEngine engine = XCard.getEngineInstance();

	    Entry entry = engine.getCurrentEntry();
	    if (entry != null) {
		boolean visited = entry.beenVisited();
		if (visited == false) {
		    // XX handle the loop
		    CardList clist = engine.getCardList();

		    if (progressValue >= (clist.size() - 1)) {
			Iterator itr = clist.iterator();
			while (itr.hasNext()) {
			    Entry en = (Entry) itr.next();
			    en.setVisited(false);
			}
			progressValue = 0;
			
		    } else {
			progressValue = (progressValue + 1) % clist.size();
			entry.setVisited(true);
		    }
		    progress.setValue(progressValue);
		}
		
		String comp = null;
		PreferencesService ps = XCard.getPreferencesService();
		String transdir = ps.getTranslationDirection();
		if (transdir.equals("fr")) {
		    comp = entry.getFullWord();
		} else {
		    comp = entry.getFullTranslation();
		}
		    
		if (translation.equals(comp)) {
		    AudioMaster audio =  XCard.getAudio();
		    if (audio != null) {
			audio.playClip("CORRECT");
		    }
		    correctResponse(entry);
		    nextEntry();
		} else {
		    incorrectResponse(entry);
		}
		updateTotalValues();
	    }
	}
    }

    private void setDisplayValues(Entry entry)
    {
	if (entry != null) {
	    PreferencesService ps = XCard.getPreferencesService();
	    String transdir = ps.getTranslationDirection();

	    word.setForeground(defaultColor);
	    
	    if (transdir.equals("fr")) {
		word.setText(entry.getFullTranslation());
	    } else {
		word.setText(entry.getFullWord());
	    }
	    translationText.setText("");
	    setScoreImage(entry);
	    comment.setText(entry.getComment());
	} else {
	    word.setText("");
	    translationText.setText("");
	    comment.setText("");
	}
    }

    public Entry nextEntry()
    {
	CardEngine engine = XCard.getEngineInstance();

	// If the user has missed 4 words.  Repeat the lesser known words
	
	if (inCorrectCounter >= 4 && nextEntryType == CardEngine.REPEATED) {
	    engine.getCardList().sortByScore();
	    engine.resetIterator();
	    inCorrectCounter = 0;
	}
	
	Entry entry = engine.getNextEntryFromCircularList(nextEntryType);
	setDisplayValues(entry);
	return entry;
    }

    private void updateTotalValues()
    {
	excellent.setText(getEntryStateCount(State.EXCELLENT));
	good.setText(getEntryStateCount(State.GOOD));
	average.setText(getEntryStateCount(State.AVERAGE));
	poor.setText(getEntryStateCount(State.POOR));
    }

    public void updateValues()
    {
	if (fieldsComposed) {
	    CardEngine engine = XCard.getEngineInstance();
	    CardList clist = engine.getCardList();
	    listName.setText(clist.getName());

	    progress.setMaximum(clist.size());
	    progressValue = engine.getNumVisitedEntries();
	    progress.setValue(progressValue);
	    
	    updateTotalValues();
	    Entry entry = engine.getCurrentEntry();
	    if (entry == null) {
		entry = engine.getNextEntryFromCircularList(nextEntryType);
	    }
	    setDisplayValues(entry);
	    translationText.requestFocus();
	} 
    }

    public void updatePreferences(PreferencesService prefs)
    {
	if (fieldsComposed) {
	    AudioMaster audio =  XCard.getAudio();
	    if (audio != null) {
		boolean enabled = prefs.isSoundEnabled();
		if (enabled) {
		    audio.enableSound();
		} else {
		    audio.disableSound();
		}
	    }
	    CardEngine engine = XCard.getEngineInstance();
	    setDisplayValues(engine.getCurrentEntry());
	}
    }

    private void buildPictureList()
    {
	try {
	    String pics = new String("pics/pics.xml");
	    Class c = getClass();
	    ClassLoader loader = c.getClassLoader();
	    URL picsXML = loader.getResource(pics);
	    
	    DocumentBuilderFactory docBuilderFactory = 
		DocumentBuilderFactory.newInstance();
	    docBuilderFactory.setValidating(false);
	    DocumentBuilder docBuilder = 
		docBuilderFactory.newDocumentBuilder();
	    Document doc = docBuilder.parse(picsXML.openStream());
	    NodeList nlist = doc.getElementsByTagName("img");
	    for (int index = 0; index < nlist.getLength(); index++) {
		Node node = nlist.item(index);
		assignPictureValue(node);
	    }

	} catch (Exception e) {
	    Message.errorMessage(e);
	}
    }

    private void assignPictureValue(Node node)
    {
	NamedNodeMap attrlist = node.getAttributes();
	if (attrlist != null) {
	    String title = null, src = null;
	    Node titleNode = attrlist.getNamedItem("title");
	    if (titleNode != null) {
		title = titleNode.getNodeValue();
	    }

	    Node srcNode = attrlist.getNamedItem("src");
	    if (srcNode != null) {
		src = srcNode.getNodeValue();
	    } else {
		System.out.println("Unable to find: " + title);
	    }
	    Picture picture = new Picture(title, src);
	    pictures.add(picture);
	}
    }

    private class Picture {
	String title;
	String src;

	Picture(String title, String src)
	{
	    this.title = title;
	    this.src = src;
	}
	
	public String getSrc()
	{
	    return src;
	}

	public String getTitle()
	{
	    return title;
	}
    }

    private Picture pick_a_picture()
    {
	Random rand = new Random();
	int random = 0;
	if (pictures.size() > 0) {
	    random = rand.nextInt(pictures.size());
	}
	return (Picture) pictures.get(random);
    }

    public void updateLocalizedText()
    {
	if (fieldsComposed) {
	    PreferencesService ps = XCard.getPreferencesService();
	    ResourceBundle resources = ps.getResourceBundle();
	    challengeLabel.setText(resources.getString("WORD:"));
	    translation.setText(resources.getString("TRANSLATION:"));
	    title.setText(resources.getString("TRANSLATE"));
	    commentLabel.setText(resources.getString("COMMENT:"));
	    next.setText(resources.getString("SKIP"));
	    checkAnswer.setText(resources.getString("CHECK_ANSWER"));
	    repeated.setText(resources.getString("REPEAT_LESSER"));
	    total.setText(resources.getString("TOTALS:"));
	}
    }
}
