/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000, 2001 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/XCardPreferences.java,v 1.12 2003/12/08 00:06:15 ellis Exp $
 */

package xcard;

import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.plaf.metal.*;


public class XCardPreferences extends JPanel {
    private ButtonGroup localeGroup = new ButtonGroup();
    private ButtonGroup transDirGroup = new ButtonGroup();
    private JRadioButton frenchButton, JRadioButton, englishButton, 
	transLationFrButton, transLationEnButton;;
    private JComboBox lookAndFeel;

    private JPanel lfPanel = null;
    private JPanel soundPanel = null;
    private JPanel localePanel = null;
    private JPanel transPanel = null;

    PreferencesService ps = null;
    ResourceBundle resources = null;

    public XCardPreferences(PreferencesService ps)
    {
	this.ps = ps;
	resources = ps.getResourceBundle();
	UIManager.addPropertyChangeListener(new UIListener(this));

	setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
	soundPanel = composeSoundPanel();
	add(Box.createVerticalStrut(10));
	localePanel = composeLocalePanel();
	transPanel = composeTransDirection();

	if (! Resource.isApple()) {
	    lfPanel = composeLookAndFeel();
	}
    }
    
    private JPanel composeSoundPanel()
    {
	JPanel soundPanel = new JPanel();
	soundPanel.setBorder(BorderFactory.createTitledBorder(resources.getString("SOUND:")));

	int volume = ps.getVolume();
	
	JSlider slider = new JSlider(0, 10, volume);
	slider.setMajorTickSpacing(1);
	slider.setSnapToTicks(true);
	slider.setPaintLabels(true);

	slider.addChangeListener(new ChangeListener() {

		public void stateChanged(ChangeEvent e) {
		    JSlider source = (JSlider)e.getSource();
		    if (!source.getValueIsAdjusting()) {
			int v = (int) source.getValue();
			AudioMaster audio = XCard.getAudio();
			audio.setVolume(v);
			audio.playClip("CORRECT");
			ps.setVolume(v);
		    }
                }
            }
				 );
		

	soundPanel.add(slider);

	add(soundPanel);
	return soundPanel;
    }

    private JPanel composeLookAndFeel()
    {
	String[] items = {"Default", "Kunststoff"};
	JPanel panel = new JPanel();
	
	panel.setBorder(BorderFactory.createTitledBorder(resources.getString("LNF:")));
	lookAndFeel = new JComboBox(items);
	String currentFeel = ps.getLookAndFeel();
	if (currentFeel != null) {
	    lookAndFeel.setSelectedItem(currentFeel);
	}
	lookAndFeel.addActionListener( new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    String item = (String) lookAndFeel.getSelectedItem();

		    try {
			ps.setLookAndFeel(item);
			Resource res = new Resource();
			if (! res.isWebStartDeployed()) {
			    if (item.equals("Kunststoff")) {
				UIManager.setLookAndFeel(new com.incors.plaf.kunststoff.KunststoffLookAndFeel());
			    
			    } else if (item.equals("Default")) {
				String name = UIManager.getSystemLookAndFeelClassName();
				UIManager.setLookAndFeel(name);
				MetalLookAndFeel.setCurrentTheme(new javax.swing.plaf.metal.DefaultMetalTheme());
				
			    }
			} else {
			    String msg = "Changing look and feel requires " +
				"a restart of XCard";
			    JOptionPane.showMessageDialog(null, msg, "Info",
							  JOptionPane.INFORMATION_MESSAGE);

			}
			
		    } catch(Exception err) {
			err.printStackTrace();
		    }
		}
	    });
				       

	panel.add(lookAndFeel);
	add(panel);
	return panel;
    }

    private JPanel composeLocalePanel()
    {
	//ResourceBundle resources = ps.getResourceBundle();
	frenchButton = new JRadioButton(resources.getString("FRENCH"));
	englishButton = new JRadioButton(resources.getString("ENGLISH"));

	JPanel localePanel = new JPanel();
	//localePanel.setPreferredSize(new Dimension(200, 50));
	localePanel.setBorder(BorderFactory.createTitledBorder(resources.getString("DISPLAY_LANG:")));

	if (ps.isLangFrench()) {
	    frenchButton.setSelected(true);
	} else {
	    englishButton.setSelected(true);
	}

	frenchButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    changeLocale(new Locale("fr", "fr"));
		}
	    });

	englishButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    changeLocale(new Locale("en", "US"));
		}
	    });

	localePanel.add(frenchButton);
	localePanel.add(englishButton);

	localeGroup.add(frenchButton);
	localeGroup.add(englishButton);

	add(localePanel);
	return localePanel;
    }

    public void changeTranslationDirection(String dir)
    {
	ps.setTranslationDirection(dir);
	firePropertyChange("PREFERENCES_TRANSDIR_CHANGE", null, this);
    }	
    
    private void changeLocale(Locale locale)
    {
	ps.setLocale(locale);
	updateLocalizedText();
	firePropertyChange("PREFERENCES_LOCALE_CHANGE", null, this);
    }

    private JPanel composeTransDirection()
    {
	transLationFrButton = new JRadioButton(resources.getString("FRENCH"));
	transLationEnButton = new JRadioButton(resources.getString("ENGLISH"));

	JPanel transPanel = new JPanel();
	//transPanel.setPreferredSize(new Dimension(200, 50));
	transPanel.setBorder(BorderFactory.createTitledBorder(resources.getString("TRANSLATE_TO:")));
	
	if (ps.getTranslationDirection().equals("fr")) {
	    transLationFrButton.setSelected(true);
	} else {
	    transLationEnButton.setSelected(true);
	}
	
	transLationFrButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    changeTranslationDirection("fr");
		}
	    });

	transLationEnButton.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
		    changeTranslationDirection("en");
		}
	    });
	
	transPanel.add(transLationFrButton);
	transPanel.add(transLationEnButton);

	transDirGroup.add(transLationFrButton);
	transDirGroup.add(transLationEnButton);

	add(transPanel);
	return transPanel;
    }


    public void fireUpdateEvent()
    {
	try {
	    ps.commitChanges();
	} catch (IOException e) {
	    Message.errorMessage(e);
	}
	firePropertyChange("PREFERENCES_UPDATE", null, this);
    }

    public void updateLocalizedText()
    {
	ResourceBundle resources = ps.getResourceBundle();
	frenchButton.setText(resources.getString("FRENCH"));
	englishButton.setText(resources.getString("ENGLISH"));

	transLationFrButton.setText(resources.getString("FRENCH"));
	transLationEnButton.setText(resources.getString("ENGLISH"));

	if (lfPanel != null) {
	    lfPanel.setBorder(BorderFactory.createTitledBorder(resources.getString("LNF:")));
	}
	soundPanel.setBorder(BorderFactory.createTitledBorder(resources.getString("SOUND:")));

	localePanel.setBorder(BorderFactory.createTitledBorder(resources.getString("DISPLAY_LANG:")));
	
	transPanel.setBorder(BorderFactory.createTitledBorder(resources.getString("TRANSLATE_TO:")));

    }

    public void updateTranslateTo()
    {
	if (ps != null && ps.getTranslationDirection().equals("fr")) {
	    transLationFrButton.setSelected(true);
	} else {
	    transLationEnButton.setSelected(true);
	}
    }

}
