/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/XCardProgressPanel.java,v 1.8 2002/11/21 04:40:40 ellis Exp $
 */

package xcard;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import com.jrefinery.data.*;
import com.jrefinery.chart.*;
import com.jrefinery.chart.ui.*;

public class XCardProgressPanel extends JPanel {
    private JFreeChart chart = null;
    ChartPanel chartPanel = null;

    public void composeChart()
    {
	XYDataset xyData = extractData();
	if (xyData != null) {
	    CardEngine engine = XCard.getEngineInstance();
	    CardList clist = engine.getCardList();
	    String name = clist.getName();
	    
	    PreferencesService ps = XCard.getPreferencesService();
	    ResourceBundle resources = ps.getResourceBundle();
	    
	    chart = ChartFactory.createTimeSeriesChart(name, "", 
						       resources.getString("COUNT")
						       ,xyData, true);
	    
	    chartPanel = new ChartPanel(chart);
	    chartPanel.setPreferredSize(new Dimension(480, 400));
	    chartPanel.setBorder(
				 BorderFactory.createCompoundBorder
				 (BorderFactory.createEmptyBorder(4, 4, 4, 4),
				  BorderFactory.createLineBorder(Color.darkGray, 
								 1)));
	    add(chartPanel);
	}
    }

    public void updateValues()
    {
	if (chart == null) {
	    composeChart();
	} else {
	    /* XXX At some point I need to just keep updating the chart rather
	       than re-building it

	    String title = engine.getName();
	    if (title != null) {
		chart.setTitle(title);
	    }
	    chart.setDataset(extractData());
	    chart.fireChartChanged();
	    */

	    CardEngine engine = XCard.getEngineInstance();
	    CardList list = engine.getCardList();
	    if (list != null && list.size() != 0) {
		removeAll();
		composeChart();
	    }
	}
    }

    public XYDataset extractData()
    {
	DefaultXYDataset dataset = null;
	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();

	String[] series = {resources.getString("EXCELLENT"),
			   resources.getString("GOOD"), 
			   resources.getString("AVERAGE"),
			   resources.getString("POOR")
	};
	int[] states = {State.EXCELLENT, State.GOOD, State.AVERAGE,
			State.POOR};
	
	CardEngine engine = XCard.getEngineInstance();
	ArrayList sessions = engine.getSessions();
	SessionEntry[] sessionArray = 
	    (SessionEntry[]) sessions.toArray(new SessionEntry[0]);

	Object[][][] data = new Object[states.length][sessionArray.length][2];

	for (int stateIndex = 0; stateIndex < states.length; stateIndex++) {
	    
	    for (int index = 0; index < sessionArray.length; index++) {
		SessionEntry sentry = sessionArray[index];
		Date date = sentry.getDate();

		int value = sentry.getSessionScore(states[stateIndex]);
		
		data[stateIndex][index][0] = date;
		data[stateIndex][index][1] = new Integer(value);
	    }
	}

	if (sessionArray.length > 0) {
	    dataset = new DefaultXYDataset(series, data);
	} 
	return dataset;
    }

    public void updateLocalizedText()
    {
	if (chart != null) {
	    updateValues();
	}
    }
}
