/* XCard
 * Author: Shawn Ellis
 * (C) Copyright 2000 
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * $Header: /home/ellis/source/java/xcard/RCS/XCardWordListPanel.java,v 1.17 2003/12/08 00:06:15 ellis Exp $
 */

package xcard;

import java.io.*;
import java.net.URL;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

public class XCardWordListPanel extends JPanel {
    java.util.List items = new ArrayList();
    JList itemList = null;

    JLabel wordListLabel = null;
    JLabel instructions = null;
    JLabel or = null;

    JButton button = null;

    public XCardWordListPanel()
    {
	getWordLists();
	composeWordLists();
    }

    private void getWordLists()
    {
	try {
	    Resource resource = new Resource();
	    String wordlist = resource.getWordProperty("wordLists");
	    StringTokenizer stok = new StringTokenizer(wordlist, ",");
	    String str = null;
	    while (stok.hasMoreTokens()) {
		str = stok.nextToken();
		if (str.endsWith(".xml")) {
		    str = str.substring(0, str.length() - 4);
		    str = str.trim();
		}
		items.add(str);
	    }
	} catch (IOException e) {
	    Message.errorMessage(e);
	}
    }

    private void composeWordLists()
    {
	JPanel panel = new JPanel();

	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();

	panel.setLayout(gbl);



	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();
	
	wordListLabel = new JLabel(resources.getString("WORD_LIST_SELECTION"));
	Font font = new Font("Helvetica", Font.BOLD + Font.ITALIC, 16);
	wordListLabel.setFont(font);
	JSeparator sep = new JSeparator();
	
	// row 1
	gbc.anchor = GridBagConstraints.NORTHWEST;
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	panel.add(wordListLabel, gbc);
	
	gbc.anchor = GridBagConstraints.WEST;
	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.insets = new Insets(0, 0, 10, 0);
	panel.add(sep, gbc);

	// row 2
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	instructions = new JLabel(resources.getString("PLEASE_CHOOSE_LIST"));
	panel.add(instructions, gbc);

	// row 3
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	itemList = new JList(items.toArray());
	ListSelectionModel selectModel = itemList.getSelectionModel();
	selectModel.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	itemList.setFixedCellWidth(320);
	itemList.setVisibleRowCount(20);
	itemList.addMouseListener(new MouseAdapter() {
		public void mouseClicked(MouseEvent e) {
		    if (e.getClickCount() == 2) {
			JList list = (JList) e.getSource();
			ListModel model = list.getModel();
			
			int index = list.locationToIndex(e.getPoint());
			String item = (String) model.getElementAt(index);
			
			boolean load_words = true;
			CardEngine engine = XCard.getEngineInstance();
			if (engine.isCardListModified()) {
			    Message message = new Message();
			    int saved = message.showUnSavedMessage(engine);

			    if (saved == Message.NO) {
				load_words = true;
			    } else if (saved == Message.YES) {
				load_words = false;
			    } else if (saved == Message.CANCEL) {
				load_words = false;
			    }
			} 

			if (load_words) {
			    loadWords(item);
			}
		    }
		}
	    });

	JScrollPane sp = new JScrollPane(itemList);
	panel.add(sp, gbc);

	gbc.fill = 0;
	gbc.anchor = GridBagConstraints.CENTER;
	button = new JButton(resources.getString("SELECT"));

	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e)
		{
		    int index = itemList.getLeadSelectionIndex();
		    if (index >= 0) {
			ListModel model = itemList.getModel();
			String item = (String) model.getElementAt(index);

			boolean load_words = true;
			CardEngine engine = XCard.getEngineInstance();
			if (engine.isCardListModified()) {
			    Message message = new Message();
			    int saved = message.showUnSavedMessage(engine);

			    if (saved == Message.NO) {
				load_words = true;
			    } else if (saved == Message.YES) {
				load_words = false;
			    } else if (saved == Message.CANCEL) {
				load_words = false;
			    }
			} 

			if (load_words) {
			    loadWords(item);
			}
		    }
		}
	    });
	panel.add(button, gbc);
	
	// row 4
	or = new JLabel(resources.getString("OR_OPEN"));

	panel.add(or, gbc);

	
	add(panel);
    }

    private void loadWords(String list) 
    {
	try {
	    // This hack is here because j2sdk1.4.x has screwed up handling
	    // files with 8 bit characters.  This is supposed to be fixed
	    // in 1.5.0.  The workaround is to have the files not use
	    // accents, but display accents on the screen

	    if (list != null) {
		if (list.equals("le médecin")) {
		    list = "le medecin";
		} else if (list.equals("irregular-verbs-passé-composé")) {
		    list = "irregular-verbs-passe-compose";
		}
	    }
	    String file = new String("words/" + list + ".xml");
	    CardEngine engine = XCard.getEngineInstance();
	    Class c = engine.getClass();
	    ClassLoader loader = c.getClassLoader();
	    URL wordList = loader.getResource(file);
	    engine.load(wordList.openStream());
	    engine.getCardList().randomize();
	    firePropertyChange("ENGINE_LOAD", null, engine);

	} catch(SAXParseException error) {
	    System.err.println("Parring error line: " + 
			       error.getLineNumber());
	    System.err.println(error.getMessage());
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public void updateLocalizedText()
    {
	PreferencesService ps = XCard.getPreferencesService();
	ResourceBundle resources = ps.getResourceBundle();

	wordListLabel.setText(resources.getString("WORD_LIST_SELECTION"));
	instructions.setText(resources.getString("PLEASE_CHOOSE_LIST"));
	or.setText(resources.getString("OR_OPEN"));
	button.setText(resources.getString("SELECT"));
    }
}
